﻿using Dss.Api.Examples.Internal;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading;
using DataScope.Select.Api;
using DataScope.Select.Api.Content;
using DataScope.Select.Api.Extractions;
using DataScope.Select.Api.Extractions.ReportExtractions;
using DataScope.Select.Api.Extractions.ReportTemplates;
using DataScope.Select.Api.Extractions.Schedules;
using DataScope.Select.Api.Extractions.SubjectLists;

// Since the samples use UtcNow, be sure your system clock is not too far away from reality.
// You can find file-content examples in the Dump() code.
namespace Dss.Api.Examples.Extractions.ReportExtractions
{

    [Example("Scheduled Extractions")]
    public class ReportExtractionExamples
    {
        protected ExtractionsContext ExtractionsContext { get; set; }

        public ReportExtractionExamples()
        {
            ExtractionsContext = ContextHelper.CreateExtractionsContext();
        }

        #region Get Available Fields

        [Example("Get Available Fields")]
        public void GetAvailableFields()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EndOfDayPricing);

            //Output
            foreach (var contentFieldType in availableFields.Take(10))
                Status.WriteLine("code: {0} / name: {1} / format type: {2}",
                    contentFieldType.Code, contentFieldType.Name, contentFieldType.FormatType);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Various Get methods

        [Example("Get By Id")]
        public void GetById()
        {
            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create the instrument list
            var instrumentList = new InstrumentList { Name = "GetById" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new[]
            {
                new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
                new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
            };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplates", "Create");
            //Define a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "GetById" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues
                //NOTE: There are many other fields which can be use to customize the report template
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "RIC", "CUSIP", "Ticker", "ISIN", "SEDOL", "Asset Category", "Currency Code", "Universal Close Price",
                "Universal Close Price Date");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedules", "Create");
            //Schedule the extraction to run immediately
            var schedule = new Schedule
            {
                Name = "GetById" + Suffix.New(),
                TimeZone = TimeZoneInfo.Local.Id,
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow, true),
                Trigger = ScheduleTrigger.CreateImmediateTrigger(true),
                ListId = instrumentList.ListId,
                ReportTemplateId = reportTemplate.ReportTemplateId,
            };
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportExtraction", "GetNextExtractionForScheduleId", MethodType.Operation, Publish.Primary);
            //Get the next extraction.  That way we have the Id which we can use later to see if it's ready to extract
            var extraction = ExtractionsContext.ReportExtractionOperations.GetNextExtractionForScheduleId(schedule.ScheduleId);

            //Output some extraction information
            Status.WriteLine("status: {0} / detailed status: {1} / extraction date: {2} - {3} / files = {4}",
                extraction.Status, extraction.DetailedStatus, extraction.ExtractionStartUtc, extraction.ExtractionEndUtc,
                extraction.Files.Count);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportExtraction", "Get", MethodType.Operation, Publish.Primary);
            //Wait a little while
            Thread.Sleep(10000);

            //Get the extraction by it's Id
            extraction = ExtractionsContext.ReportExtractionOperations.Get(extraction.ReportExtractionId);

            //Output some extraction information
            Status.WriteLine("status: {0} / detailed status: {1} / extraction date: {2} - {3} / files = {4}",
                extraction.Status, extraction.DetailedStatus, extraction.ExtractionStartUtc, extraction.ExtractionEndUtc,
                extraction.Files.Count);
            Status.EndNotify(ExtractionsContext);

            //Delete the extracted files, schedule, instrument list and report template.  See the "Cleanup" example for
            //the various steps involved with cleanup.  NOTE: It is a good idea to clean up the extracted files (Report Extractions)
            //once you have downloaded the results.  If the schedule is not immediate (i.e. recurring), you want to keep the 
            //instrument list, report template and schedule around.
            Cleanup(schedule, instrumentList, reportTemplate);
        }

        [Example("Get Completed By Date Range")]
        public void GetCompletedByDateRange()
        {
            /* 
             * GetCompletedByDateRange is best suited for use with scheduled extractions.  If you have multiple schedules that
             * run daily, you can use this method to find completed extractions for the day.
             */
            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create the instrument list
            var instrumentList = new InstrumentList { Name = "GetCompletedByDateRange" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new[]
            {
                new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
                new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
            };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplates", "Create");
            //Define a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "GetCompletedByDateRange" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues
                //NOTE: There are many other fields which can be use to customize the report template
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "RIC", "CUSIP", "Ticker", "ISIN", "SEDOL", "Asset Category", "Currency Code", "Universal Close Price",
                "Universal Close Price Date");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedules", "Create");
            //Schedule the extraction to run immediately
            var schedule = new Schedule
            {
                Name = "GetCompletedByDateRange" + Suffix.New(),
                TimeZone = TimeZoneInfo.Local.Id,
                //GetCompletedByDateRange would make more sense to use with a recurring Schedule rather than an immediate
                //schedule, but for the purposes of this example, we're just using an immediate schedule.
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow, true),
                Trigger = ScheduleTrigger.CreateImmediateTrigger(true),
                ListId = instrumentList.ListId,
                ReportTemplateId = reportTemplate.ReportTemplateId,
            };
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportExtraction", "GetLastExtractionForScheduleId");
            //Wait for the extraction to complete so that we have something to retrieve with GetCompletedByDateRange.
            ExtractionsContext.ScheduleOperations.WaitForNextExtraction(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportExtraction", "GetCompletedByDateRange", MethodType.Operation, Publish.Primary);
            //Get the extraction by it's Id
            var completedExtractions = ExtractionsContext.ReportExtractionOperations.GetCompletedByDateRange(
                DateTime.UtcNow.Date, DateTime.UtcNow.AddMinutes(5));

            //Output some extraction information
            foreach (var extraction in completedExtractions)
                Status.WriteLine("schedule name: {0} / status: {1} / extraction date: {2} - {3}",
                    extraction.ScheduleName, extraction.Status, extraction.DetailedStatus,
                    extraction.ExtractionStartUtc, extraction.ExtractionEndUtc);
            Status.EndNotify(ExtractionsContext);

            //Delete the extracted files, schedule, instrument list and report template.  See the "Cleanup" example for
            //the various steps involved with cleanup.  NOTE: It is a good idea to clean up the extracted files (Report Extractions)
            //once you have downloaded the results.  If the schedule is not immediate (i.e. recurring), you want to keep the 
            //instrument list, report template and schedule around.
            Cleanup(schedule, instrumentList, reportTemplate);
        }

        [Example("Get Completed By Date Range By Schedule Id")]
        public void GetCompletedByDateRangeByScheduleId()
        {
            /* 
             * GetCompletedByDateRange is best suited for use with scheduled extractions.  If you have multiple schedules that
             * run daily, you can use this method to find completed extractions for the day.
             */
            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create the instrument list
            var instrumentList = new InstrumentList { Name = "GetCompletedByDateRangeByScheduleId" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new[]
            {
                new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
                new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
            };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplates", "Create");
            //Define a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "GetCompletedByDateRangeByScheduleId" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues
                //NOTE: There are many other fields which can be use to customize the report template
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "RIC", "CUSIP", "Ticker", "ISIN", "SEDOL", "Asset Category", "Currency Code", "Universal Close Price",
                "Universal Close Price Date");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedules", "Create");
            //Schedule the extraction to run immediately
            var schedule = new Schedule
            {
                Name = "GetCompletedByDateRangeByScheduleId" + Suffix.New(),
                TimeZone = TimeZoneInfo.Local.Id,
                //GetCompletedByDateRange would make more sense to use with a recurring Schedule rather than an immediate
                //schedule, but for the purposes of this example, we're just using an immediate schedule.
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow, true),
                Trigger = ScheduleTrigger.CreateImmediateTrigger(true),
                ListId = instrumentList.ListId,
                ReportTemplateId = reportTemplate.ReportTemplateId,
            };
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportExtraction", "GetLastExtractionForScheduleId");
            //Wait for the extraction to complete so that we have something to retrieve with GetCompletedByDateRange.
            ExtractionsContext.ScheduleOperations.WaitForNextExtraction(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportExtraction", "GetCompletedByDateRangeByScheduleId", MethodType.Operation, Publish.Primary);
            //Get the extraction by it's Id
            var completedExtractions = ExtractionsContext.ReportExtractionOperations.GetCompletedByDateRangeByScheduleId(
                schedule.ScheduleId, DateTime.UtcNow.Date, DateTime.UtcNow.AddMinutes(5));

            //Output some extraction information
            foreach (var extraction in completedExtractions)
                Status.WriteLine("schedule name: {0} / status: {1} / extraction date: {2} - {3}",
                    extraction.ScheduleName, extraction.Status, extraction.DetailedStatus,
                    extraction.ExtractionStartUtc, extraction.ExtractionEndUtc);
            Status.EndNotify(ExtractionsContext);

            //Delete the extracted files, schedule, instrument list and report template.  See the "Cleanup" example for
            //the various steps involved with cleanup.  NOTE: It is a good idea to clean up the extracted files (Report Extractions)
            //once you have downloaded the results.  If the schedule is not immediate (i.e. recurring), you want to keep the 
            //instrument list, report template and schedule around.
            Cleanup(schedule, instrumentList, reportTemplate);
        }

        #endregion

        #region Immediate (not customized)

        [Example("Immediate")]
        public void ImmediateNotCustomized()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EndOfDayPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create the instrument list
            var instrumentList = new InstrumentList { Name = "ImmediateNonCustomized" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new[]
            {
                new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
                new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
            };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplates", "Create");
            //Define a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "ImmediateNonCustomized" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues
                //NOTE: There are many other fields which can be use to customize the report template
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "RIC", "CUSIP", "Ticker", "ISIN", "SEDOL", "Asset Category", "Currency Code", "Universal Close Price",
                "Universal Close Price Date");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedules", "Create");
            //Schedule the extraction to run immediately
            var schedule = new Schedule
            {
                Name = "ImmediateNonCustomized" + Suffix.New(),
                TimeZone = TimeZoneInfo.Local.Id,
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow, true),
                Trigger = ScheduleTrigger.CreateImmediateTrigger(true),
                ListId = instrumentList.ListId,
                ReportTemplateId = reportTemplate.ReportTemplateId,
            };
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            try
            {
                Status.Notify(ExtractionsContext, "ReportExtraction", "GetLastExtractionForScheduleId", MethodType.Operation, Publish.Primary);
                //Wait for the extraction to complete.  This involves polling GetLastExtractionForScheduleId until 
                //the returned ReportExtraction.Status is completed.
                var extraction = ExtractionsContext.ScheduleOperations.WaitForNextExtraction(schedule);

                //Output some extraction information
                Status.WriteLine("status: {0} / detailedstatus: {1} / extraction date: {2} - {3} / files = {4}",
                    extraction.Status, extraction.DetailedStatus, extraction.ExtractionStartUtc, extraction.ExtractionEndUtc,
                    extraction.Files.Count);
                Status.EndNotify(ExtractionsContext);

                Status.Notify(ExtractionsContext, "ReportExtraction", "Files", MethodType.Association, Publish.Primary);
                //Get the list of files for this extraction.  This includes the contents, notes files and RIC maintenance files.
                ExtractionsContext.LoadProperty(extraction, "Files");

                //Find the extract file.  Other ExtractedFileTypes are: Note, RicMaintenanceNote, Full, Partial, Other
                var extractFile = extraction.Files.FirstOrDefault(f => f.FileType == ExtractedFileType.Full);
                Status.EndNotify(ExtractionsContext);

                if (extractFile != null)
                {
                    Status.Notify(ExtractionsContext, "ReportExtraction", "GetReadStream", MethodType.Stream, Publish.Primary);
                    //Output the contents of the extracted file
                    var streamResponse = ExtractionsContext.ExtractedFileOperations.GetReadStream(extractFile);
                    string internalStreamOutput;
                    using (var reader = new StreamReader(streamResponse.Stream))
                    {
                        Status.WriteLine(internalStreamOutput = reader.ReadToEnd());
                    }
                    Status.EndNotify(ExtractionsContext, internalStreamOutput); // Extract the stream output and pass it as responseBody.
                }
            }
            catch (TimeoutException ex)
            {
                Status.WriteLine("Timed out: " + ex.Message);
            }

            //Delete the extracted files, schedule, instrument list and report template.  See the "Cleanup" example for
            //the various steps involved with cleanup.  NOTE: It is a good idea to clean up the extracted files (Report Extractions)
            //once you have downloaded the results.  If the schedule is not immediate (i.e. recurring), you want to keep the 
            //instrument list, report template and schedule around.
            Cleanup(schedule, instrumentList, reportTemplate);
        }

        #endregion

        #region Custom Formatting

        [Example("Custom Formatting")]
        public void CustomFormatting()
        {
            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create the instrument list
            var instrumentList = new InstrumentList { Name = "CustomFormatting" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new[]
            {
                new InstrumentIdentifier
                {
                    Identifier = "IBM.N",
                    IdentifierType = IdentifierType.Ric,
                    UserDefinedIdentifier = "My Custom Identifier 1a",
                    UserDefinedIdentifier2 = "My Custom Identifier 2a",
                    UserDefinedIdentifier3 = "My Custom Identifier 3a",
                },
                new InstrumentIdentifier
                {
                    Identifier = "KO.N",
                    IdentifierType = IdentifierType.Ric,
                    UserDefinedIdentifier = "My Custom Identifier 1b",
                    UserDefinedIdentifier2 = "My Custom Identifier 2b",
                    UserDefinedIdentifier3 = "My Custom Identifier 3b",
                }
            };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplates", "Create");
            //Define a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "CustomFormatting" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues,
                CompressionType = ReportCompressionType.None,
                Delimiter = ReportDelimiter.Pipe,
                DeliveryType = ReportDeliveryType.None,
                Destination = null,
            };

            //Add simple fields (without custom formatting)
            reportTemplate.ContentFields.AddRange(
                "RIC", "CUSIP", "Ticker", "ISIN", "SEDOL", "Asset Category", "Currency Code");

            reportTemplate.ContentFields.Add(new ContentField
            {
                FieldName = "Universal Close Price",
                Justification = ContentFieldJustification.Left,
                Label = "Close Price",
                Width = 10,
                WidthStyle = ContentFieldWidthStyle.FixedWidth,
                //NOTE: If custom formatting is not providedm the default user preferences will be applied.
                Format = new ContentFieldNumberFormat
                {
                    DecimalPlaces = 2,
                    DecimalSeparator = ContentFieldDecimalSeparator.Period,
                    IntegerPlaces = 4,
                    NegativeSignPosition = ContentFieldNegativeSignPosition.Before,
                    ThousandSeparator = ContentFieldThousandsSeparator.Comma,
                    UseLeadingZero = false,
                    UseThousandSeparator = true,
                    UseTrailingZero = true,
                }
            });
            reportTemplate.ContentFields.Add(new ContentField
            {
                FieldName = "Universal Close Price Date",
                Justification = ContentFieldJustification.Left,
                Label = "Close Date",
                Width = 10,
                WidthStyle = ContentFieldWidthStyle.FixedWidth,
                Format = new ContentFieldDateFormat
                {
                    DateFormat = "dd/MM/yyyy"
                }
            });

            //Add headers and trailers
            reportTemplate.Headers.Add(new ReportHeaderField
            {
                LineNumber = 1,
                Label = "First",
                Number = 1,
                FieldType = ReportHeaderFieldType.RowCount,
                Justification = ContentFieldJustification.Left,
                WidthStyle = ContentFieldWidthStyle.VariableWidth,
                ShowThousandsSeparator = true,
            });

            reportTemplate.Headers.Add(new ReportHeaderField
            {
                LineNumber = 1,
                Label = "Second",
                Number = 2,
                FieldType = ReportHeaderFieldType.Timezone,
                Justification = ContentFieldJustification.Left,
                WidthStyle = ContentFieldWidthStyle.VariableWidth,
            });

            reportTemplate.Trailers.Add(new ReportHeaderField
            {
                LineNumber = 1,
                Label = "Trailer Label",
                Number = 3,
                FieldType = ReportHeaderFieldType.User,
                Justification = ContentFieldJustification.Left,
                WidthStyle = ContentFieldWidthStyle.FixedWidth,
                UserData = "Custom",
                Width = 20
            });

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedules", "Create");
            //Schedule the extraction to run immediately
            var schedule = new Schedule
            {
                Name = "CustomFormatting" + Suffix.New(),
                TimeZone = TimeZoneInfo.Local.Id,
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow, true),
                Trigger = ScheduleTrigger.CreateImmediateTrigger(true),
                ListId = instrumentList.ListId,
                ReportTemplateId = reportTemplate.ReportTemplateId,
            };
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            try
            {
                Status.Notify(ExtractionsContext, "ReportExtraction", "GetLastExtractionForScheduleId");
                //Wait for the extraction to complete.  This involves polling GetLastExtractionForScheduleId until 
                //the returned ReportExtraction.Status is completed.
                var extraction = ExtractionsContext.ScheduleOperations.WaitForNextExtraction(schedule);

                //Output some extraction information
                Status.WriteLine("status: {0} / detailedstatus: {1} / extraction date: {2} - {3} / files = {4}",
                    extraction.Status, extraction.DetailedStatus, extraction.ExtractionStartUtc, extraction.ExtractionEndUtc,
                    extraction.Files.Count);
                Status.EndNotify(ExtractionsContext);

                Status.Notify(ExtractionsContext, "ReportExtraction", "Files");
                //Get the list of files for this extraction.  This includes the contents, notes files and RIC maintenance files.
                ExtractionsContext.LoadProperty(extraction, "Files");

                //Find the extract file.  Other ExtractedFileTypes are: Note, RicMaintenanceNote, Full, Partial, Other
                var extractFile = extraction.Files.FirstOrDefault(f => f.FileType == ExtractedFileType.Full);
                Status.EndNotify(ExtractionsContext);

                if (extractFile != null)
                {
                    Status.Notify(ExtractionsContext, "ReportExtraction", "GetReadStream");
                    //Output the contents of the extracted file
                    var streamResponse = ExtractionsContext.ExtractedFileOperations.GetReadStream(extractFile);
                    string internalStreamOutput;
                    using (var reader = new StreamReader(streamResponse.Stream))
                    {
                        Status.WriteLine(internalStreamOutput = reader.ReadToEnd());
                    }
                    Status.EndNotify(ExtractionsContext, internalStreamOutput); // Extract the stream output and pass it as responseBody.
                }
            }
            catch (TimeoutException ex)
            {
                Status.WriteLine("Timed out: " + ex.Message);
            }

            //Delete the extracted files, schedule, instrument list and report template.  See the "Cleanup" example for
            //the various steps involved with cleanup.  NOTE: It is a good idea to clean up the extracted files (Report Extractions)
            //once you have downloaded the results.  If the schedule is not immediate (i.e. recurring), you want to keep the 
            //instrument list, report template and schedule around.
            Cleanup(schedule, instrumentList, reportTemplate);
        }

        #endregion

        #region Fetch All Files

        [Example("Fetch All Files")]
        public void FetchAllFiles()
        {
            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create the instrument list
            var instrumentList = new InstrumentList { Name = "FetchAllFiles" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new[]
            {
                new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
                new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
            };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplates", "Create");
            //Define a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "FetchAllFiles" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues
                //NOTE: There are many other fields which can be use to customize the report template
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "RIC", "CUSIP", "Ticker", "ISIN", "SEDOL", "Asset Category", "Currency Code", "Universal Close Price",
                "Universal Close Price Date");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedules", "Create");
            //Schedule the extraction to run immediately
            var schedule = new Schedule
            {
                Name = "FetchAllFiles" + Suffix.New(),
                TimeZone = TimeZoneInfo.Local.Id,
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow, true),
                Trigger = ScheduleTrigger.CreateImmediateTrigger(true),
                ListId = instrumentList.ListId,
                ReportTemplateId = reportTemplate.ReportTemplateId,
            };
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            try
            {
                Status.Notify(ExtractionsContext, "ReportExtraction", "GetLastExtractionForScheduleId");
                //Wait for the extraction to complete.  This involves polling GetLastExtractionForScheduleId until 
                //the returned ReportExtraction.Status is completed.
                var extraction = ExtractionsContext.ScheduleOperations.WaitForNextExtraction(schedule);

                //Output some extraction information
                Status.WriteLine("status: {0} / detailedstatus: {1} / extraction date: {2} - {3} / files = {4}",
                    extraction.Status, extraction.DetailedStatus, extraction.ExtractionStartUtc, extraction.ExtractionEndUtc,
                    extraction.Files.Count);
                Status.EndNotify(ExtractionsContext);

                Status.Notify(ExtractionsContext, "ReportExtraction", "Files", MethodType.Association, Publish.Secondary);
                //Get the list of files for this extraction.  This includes the contents, notes files and RIC maintenance files.
                ExtractionsContext.LoadProperty(extraction, "Files");
                Status.EndNotify(ExtractionsContext);

                //Store the file count so that we can tell when a new file arrives
                foreach (var file in extraction.Files)
                {
                    //Output the contents of the extracted file
                    Status.WriteLine("File Name:" + file.ExtractedFileName);
                    Status.WriteLine("File Type:" + file.FileType);
                    Status.WriteLine("Received:" + file.ReceivedDateUtc);
                    Status.WriteLine("Size:" + file.Size);
                    if (!file.ContentsExists)
                        Status.WriteLine("No Contents");
                    else
                    {
                        Status.Notify(ExtractionsContext, "ReportExtraction", "GetReadStream");
                        Status.WriteLine("=============================FILE CONTENTS=============================");
                        var streamResponse = ExtractionsContext.ExtractedFileOperations.GetReadStream(file);
                        string internalStreamOutput;
                        using (var reader = new StreamReader(streamResponse.Stream))
                        {
                            Status.WriteLine(internalStreamOutput = reader.ReadToEnd());
                        }
                        Status.EndNotify(ExtractionsContext, internalStreamOutput); // Extract the stream output and pass it as responseBody.
                    }
                }
            }
            catch (TimeoutException ex)
            {
                Status.WriteLine("Timed out: " + ex.Message);
            }

            //Delete the extracted files, schedule, instrument list and report template.  See the "Cleanup" example for
            //the various steps involved with cleanup.  NOTE: It is a good idea to clean up the extracted files (Report Extractions)
            //once you have downloaded the results.  If the schedule is not immediate (i.e. recurring), you want to keep the 
            //instrument list, report template and schedule around.
            Cleanup(schedule, instrumentList, reportTemplate);
        }

        #endregion

        #region Get Next Extraction

        [Example("Get Next Extraction")]
        public void GetNextExtraction()
        {
            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create the instrument list
            var instrumentList = new InstrumentList { Name = "GetNextExtraction" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new[]
            {
                new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
                new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
            };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplates", "Create");
            //Define a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "GetNextExtraction" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues
                //NOTE: There are many other fields which can be use to customize the report template
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "RIC", "CUSIP", "Ticker", "ISIN", "SEDOL", "Asset Category", "Currency Code", "Universal Close Price",
                "Universal Close Price Date");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedules", "Create");
            //Schedule the extraction to run immediately
            var name = "GetNextExtraction" + Suffix.New();
            var schedule = new Schedule
            {
                Name = name,
                OutputFileName = name,
                TimeZone = TimeZoneInfo.Local.Id,
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow.AddDays(1), isImmediate: false),
                Trigger = new TimeTrigger
                {
                    LimitReportToTodaysData = false,
                    At = new[] { new HourMinute { Hour = 5, Minute = 0 } }
                },
                ListId = instrumentList.ListId,
                ReportTemplateId = reportTemplate.ReportTemplateId,
            };
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportExtraction", "GetNextExtractionForScheduleId");
            //Lazy-load the next extraction for the schedule
            ExtractionsContext.LoadProperty(schedule, s => s.NextExtraction);
            var extraction = schedule.NextExtraction;
            //Output some extraction information
            Status.WriteLine("status: {0} / detailedstatus: {1} / extraction date: {2} - {3} / files = {4}",
                extraction.Status, extraction.DetailedStatus, extraction.ExtractionStartUtc, extraction.ExtractionEndUtc,
                extraction.Files.Count);
            Status.EndNotify(ExtractionsContext);

            //Delete the extracted files, schedule, instrument list and report template.  See the "Cleanup" example for
            //the various steps involved with cleanup.  NOTE: It is a good idea to clean up the extracted files (Report Extractions)
            //once you have downloaded the results.  If the schedule is not immediate (i.e. recurring), you want to keep the 
            //instrument list, report template and schedule around.
            Cleanup(schedule, instrumentList, reportTemplate);
        }

        #endregion

        #region Extract Now

        [Example("ExtractNow")]
        public void ExtractNow()
        {
            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create the instrument list
            var instrumentList = new InstrumentList { Name = "ExtractNow" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new[]
            {
                new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
                new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
            };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplates", "Create");
            //Define a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "ExtractNow" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues
                //NOTE: There are many other fields which can be use to customize the report template
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "RIC", "CUSIP", "Ticker", "ISIN", "SEDOL", "Asset Category", "Currency Code", "Universal Close Price",
                "Universal Close Price Date");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedules", "Create");
            //Schedule the extraction to run immediately
            var schedule = new Schedule
            {
                Name = "ExtractNow" + Suffix.New(),
                TimeZone = TimeZoneInfo.Local.Id,
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow.AddDays(1), isImmediate: false),
                Trigger = new TimeTrigger
                {
                    LimitReportToTodaysData = false,
                    At = new[] { new HourMinute { Hour = 5, Minute = 0 } }
                },
                ListId = instrumentList.ListId,
                ReportTemplateId = reportTemplate.ReportTemplateId,
            };
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "ExtractNow");
            //Force an extraction which does not take the schedule or triggering into consideration.  You can use ExtractNow
            //to preview the results of an extraction.
            var extraction = ExtractionsContext.ScheduleOperations.ExtractNow(schedule);

            //Output some extraction information
            Status.WriteLine("status: {0} / detailedstatus: {1} / extraction date: {2} - {3} / files = {4}",
            extraction.Status, extraction.DetailedStatus, extraction.ExtractionStartUtc, extraction.ExtractionEndUtc,
            extraction.Files.Count);
            Status.EndNotify(ExtractionsContext);

            try
            {
                Status.Notify(ExtractionsContext, "ReportExtraction", "Files", MethodType.Association);
                //Get the list of files for this extraction.  This includes the contents, notes files and RIC maintenance files.
                ExtractionsContext.LoadProperty(extraction, "Files");

                //Find the extract file.  Other ExtractedFileTypes are: Note, RicMaintenanceNote, Full, Partial, Other
                var extractFile = extraction.Files.FirstOrDefault(f => f.FileType == ExtractedFileType.Full);
                Status.EndNotify(ExtractionsContext);

                if (extractFile != null)
                {
                    Status.Notify(ExtractionsContext, "ReportExtraction", "GetReadStream", MethodType.Stream);
                    //Output the contents of the extracted file
                    var streamResponse = ExtractionsContext.ExtractedFileOperations.GetReadStream(extractFile);
                    string internalStreamOutput;
                    using (var reader = new StreamReader(streamResponse.Stream))
                    {
                        Status.WriteLine(internalStreamOutput = reader.ReadToEnd());
                    }
                    Status.EndNotify(ExtractionsContext, internalStreamOutput); // Extract the stream output and pass it as responseBody.
                }
            }
            catch (TimeoutException ex)
            {
                Status.WriteLine("Timed out: " + ex.Message);
            }

            //Delete the extracted files, schedule, instrument list and report template.  See the "Cleanup" example for
            //the various steps involved with cleanup.  NOTE: It is a good idea to clean up the extracted files (Report Extractions)
            //once you have downloaded the results.  If the schedule is not immediate (i.e. recurring), you want to keep the 
            //instrument list, report template and schedule around.
            Cleanup(schedule, instrumentList, reportTemplate);
        }

        #endregion

        #region Partial Files / Embargo

        //[Example("Partial Files / Embargo")]
        //public void PartialFiles()
        //{
        //    //Embargoed scheduled extractions can produce multiple files as the data becomes available.  This example
        //    //demonstrates how to retrieve the contents as it becomes available.

        //    Status.Notify(ExtractionsContext, "InstrumentList", "Create");

        //    var note1 = "This example may take many minutes to complete.  If you want to monitor progress, use the Visual Studio debug output window to see live debug messages. ";
        //    var note2 = "You must set your DSS user preferences to 'Enable Intermediate Embargoed Reports' for this sample to work as intended.";
        //    var note3 = "Actual files delivered may vary.  Embargoes vary during the day.  For example, there are no embargoes outside trading hours.";
        //    Status.WriteLine("NOTE: " + note1);
        //    Status.WriteLine("NOTE: " + note2);
        //    Status.WriteLine("NOTE: " + note3);

        //    MessageBox.Show("NOTES: \n\n1) " + note1 + "\n\n2) " + note2 + "\n\n3) " + note3);

        //    var instrumentList = new InstrumentList { Name = "PartialFiles" + Suffix.New() };
        //    ExtractionsContext.InstrumentListOperations.Create(instrumentList);
        //    Status.EndNotify(ExtractionsContext);

        //    Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
        //    var instruments = new List<InstrumentIdentifier>
        //    {
        //        new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
        //        new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
        //    };
        //    ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
        //    Status.EndNotify(ExtractionsContext);

        //    Status.Notify(ExtractionsContext, "ReportTemplates", "Create");
        //    //Define an Intraday report template
        //    var reportTemplate = new IntradayPricingReportTemplate
        //    {
        //        Name = "PartialFiles" + Suffix.New(),
        //        OutputFormat = ReportOutputFormat.CommaSeparatedValues,
        //        CompressionType = ReportCompressionType.None,
        //        Delimiter = ReportDelimiter.Pipe,
        //        DeliveryType = ReportDeliveryType.None,
        //    };

        //    //These fields are examples of ones that will cause an embargo.
        //    reportTemplate.ContentFields.AddRange(
        //        "Ask Price", "Ask Yield", "ISIN", "Ask Size", "Asset Category", "Asset Category Description", "Asset ID",
        //        "Asset Status", "Asset Status Description", "Asset SubType", "Asset SubType Description",
        //        "Asset Swap Spread", "Asset Type", "Asset Type Description");

        //    //Create the report template
        //    ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
        //    Status.EndNotify(ExtractionsContext);

        //    Status.Notify(ExtractionsContext, "Schedules", "Create");
        //    //Create an immediate schedule 
        //    var schedule = new Schedule
        //    {
        //        Name = "PartialFiles" + Suffix.New(),
        //        TimeZone = TimeZoneInfo.Local.Id,
        //        Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow, isImmediate: true),
        //        Trigger = new ImmediateTrigger() { LimitReportToTodaysData = false },
        //        ListId = instrumentList.ListId,
        //        ReportTemplateId = reportTemplate.ReportTemplateId,
        //    };

        //    ExtractionsContext.ScheduleOperations.Create(schedule);
        //    Status.EndNotify(ExtractionsContext);
        //    Status.WriteLine("List created, Report created, Immediate schedule created.  Next step: wait for extraction to start.  Current time UTC: {0:T}.", DateTime.UtcNow);


        //    // Force retrieval of the newly created schedule's ID.
        //    Status.Notify(ExtractionsContext, "Schedules", "GetByName");
        //    schedule = ExtractionsContext.ScheduleOperations.GetByName(schedule.Name);
        //    Status.EndNotify(ExtractionsContext);

        //    // The extraction may have started by this point.  But if server load is high at this moment, we may have to 
        //    // wait for the extraction to start.

        //    Status.Notify(ExtractionsContext, "ReportExtraction", "GetByScheduleId");
        //    ReportExtraction extraction = null;
        //    while (extraction == null)
        //    {
        //        extraction = ExtractionsContext.ReportExtractionOperations.GetByScheduleId(schedule.ScheduleId).FirstOrDefault();
        //        if (extraction == null)
        //        {
        //            Status.WriteLine("Waiting 60 seconds for server to initiate the extraction.  Current time is {0:T} UTC.", DateTime.UtcNow);
        //            Thread.Sleep(60 * 1000);
        //        }
        //    }

        //    var extractionReportId = extraction.ReportExtractionId;
        //    Status.WriteLine("Done waiting for the extraction to start.  Current time is {0:T} UTC.", DateTime.UtcNow);
        //    Status.EndNotify(ExtractionsContext);

        //    // wait for completion
        //    var retrievedFiles = new HashSet<string>();
        //    while (true)
        //    {
        //        Status.Notify(ExtractionsContext, "ReportExtraction", "Files", MethodType.Association);
        //        Status.WriteLine("Extraction Status: " + extraction.Status);
        //        Status.WriteLine("Extraction Detailed Status: " + extraction.DetailedStatus);
        //        Status.WriteLine("Current time is {0:T} UTC.", DateTime.UtcNow);

        //        // Update the list of files. If the extraction is incomplete, the Files property needs to be refreshed.  
        //        // Embargoed data may arrive while the extraction is processing.  Also, a re-Get() of the extraction 
        //        // empties the property.  When the extraction is complete, all files have been delivered. 
        //        ExtractionsContext.LoadProperty(extraction, "Files");
        //        Status.EndNotify(ExtractionsContext);

        //        var newFiles = extraction.Files.Where(f => !retrievedFiles.Contains(f.ExtractedFileName));
        //        foreach (var newFile in newFiles)
        //        {
        //            // Make a note that this file has already been retrieved
        //            retrievedFiles.Add(newFile.ExtractedFileName);
        //            Status.Notify(ExtractionsContext, "ReportExtraction", "GetReadStream");

        //            // Output the the extracted file
        //            Status.WriteLine("File Name:" + newFile.ExtractedFileName);
        //            Status.WriteLine("File Type:" + newFile.FileType);
        //            Status.WriteLine("Received:" + newFile.ReceivedDateUtc);
        //            Status.WriteLine("Size:" + newFile.Size);
        //            Status.WriteLine("ContentsExists:" + newFile.ContentsExists);

        //            // Don't fetch the content if the file is empty
        //            if (!newFile.ContentsExists || newFile.Size == 0)
        //            {
        //                Status.WriteLine("File is Empty. Skipping contents dump.");
        //                Status.EndNotify(ExtractionsContext);
        //                continue;
        //            }

        //            Status.WriteLine("=============================FILE CONTENTS=============================");
        //            var streamResponse = ExtractionsContext.ExtractedFileOperations.GetReadStream(newFile);
        //            using (var reader = new StreamReader(streamResponse.Stream))
        //                Status.WriteLine(reader.ReadToEnd());

        //            Status.EndNotify(ExtractionsContext);
        //        }

        //        if (extraction.Status == ReportExtractionStatus.Completed)
        //        {
        //            Status.WriteLine("Extraction is done. Exiting wait loop.  Current time is {0:T} UTC.", DateTime.UtcNow);
        //            break;
        //        }

        //        Status.Notify(ExtractionsContext, "ReportExtraction", "Get");

        //        // Wait for server to make progress.  Data may be embargoed &/or servers may be busy.
        //        Status.WriteLine("Waiting 60 seconds for server to make progress on the extraction.  Current time is {0:T} UTC.", DateTime.UtcNow);
        //        Thread.Sleep(60 * 1000);

        //        // Refresh extraction in order to get updated status.  Additional note: the Files property is cleared at this point.
        //        extraction = ExtractionsContext.ReportExtractionOperations.Get(extraction.ReportExtractionId);
        //        Status.EndNotify(ExtractionsContext);
        //    }

        //    Status.WriteLine("Sample complete.  All files have been delivered.  Current time is {0:T} UTC.", DateTime.UtcNow);
        //}

        #endregion

        #region Immediate (not customized)

        [Example("Cleanup")]
        public void Cleanup()
        {
            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create the instrument list
            var instrumentList = new InstrumentList { Name = "Cleanup" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new[]
            {
                new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
                new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
            };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplates", "Create");
            //Define a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "Cleanup" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues
                //NOTE: There are many other fields which can be use to customize the report template
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "RIC", "CUSIP", "Ticker", "ISIN", "SEDOL", "Asset Category", "Currency Code", "Universal Close Price",
                "Universal Close Price Date");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedules", "Create");
            //Schedule the extraction to run immediately
            var schedule = new Schedule
            {
                Name = "Cleanup" + Suffix.New(),
                TimeZone = TimeZoneInfo.Local.Id,
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow, true),
                Trigger = ScheduleTrigger.CreateImmediateTrigger(true),
                ListId = instrumentList.ListId,
                ReportTemplateId = reportTemplate.ReportTemplateId,
            };
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            try
            {
                Status.Notify(ExtractionsContext, "ReportExtraction", "GetLastExtractionForScheduleId");
                //Wait for the extraction to complete.  This involves polling GetLastExtractionForScheduleId until 
                //the returned ReportExtraction.Status is completed.
                var extraction = ExtractionsContext.ScheduleOperations.WaitForNextExtraction(schedule);

                //Output some extraction information
                Status.WriteLine("status: {0} / detailedstatus: {1} / extraction date: {2} - {3} / files = {4}",
                    extraction.Status, extraction.DetailedStatus, extraction.ExtractionStartUtc, extraction.ExtractionEndUtc,
                    extraction.Files.Count);
                Status.EndNotify(ExtractionsContext);
            }
            catch (TimeoutException ex)
            {
                Status.WriteLine("Timed out: " + ex.Message);
            }

            //Delete the extracted files, schedule, instrument list and report template.  See the "Cleanup" example for
            //the various steps involved with cleanup.  NOTE: It is a good idea to clean up the extracted files (Report Extractions)
            //once you have downloaded the results.  If the schedule is not immediate (i.e. recurring), you want to keep the 
            //instrument list, report template and schedule around.
            if (schedule.ScheduleId != null)
            {
                Status.Notify(ExtractionsContext, "ReportExtraction", "GetByScheduleId");
                //Get the extractions so that we can delete the extraction files
                var scheduleExtractions = ExtractionsContext.ReportExtractionOperations.GetByScheduleId(schedule.ScheduleId);
                Status.EndNotify(ExtractionsContext);

                foreach (var reportExtraction in scheduleExtractions)
                {
                    Status.Notify(ExtractionsContext, "ReportExtraction", "Files", MethodType.Association);
                    //Get all the files for this extraction
                    ExtractionsContext.LoadProperty(reportExtraction, "Files");
                    Status.EndNotify(ExtractionsContext);

                    Status.Notify(ExtractionsContext, "ReportExtraction", "DeleteMultiple", MethodType.Operation, Publish.Primary);
                    //Use the DeleteMultiple method to delete all the files associated with the schedule
                    var fileIds = reportExtraction.Files
                        .Where(f => f.ContentsExists).Select(f => f.ExtractedFileId).ToList();
                    if (fileIds.Count > 0)
                        ExtractionsContext.ExtractedFileOperations.DeleteMultiple(fileIds);
                    Status.EndNotify(ExtractionsContext);
                }

                Status.Notify(ExtractionsContext, "Schedule", "Delete");
                //Delete the schedule
                ExtractionsContext.ScheduleOperations.Delete(schedule);
                Status.EndNotify(ExtractionsContext);
            }

            Status.Notify(ExtractionsContext, "ReportTemplate", "Delete");
            if (reportTemplate.ReportTemplateId != null)
                ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "Delete");
            if (instrumentList.ListId != null)
                ExtractionsContext.InstrumentListOperations.Delete(instrumentList);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Cancel

        [Example("Cancel By Report Extraction (Tick History Only)")]
        public void CancelByReportExtraction()
        {
            /* See also: Schedule Examples - Cancel by Schedule 
             */
            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "example-eod" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
            {
                new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
                new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
            };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "TickHistoryTimeAndSalesReportTemplate", "Create", MethodType.Operation, Publish.Secondary);
            //Define the report template specific to the desired content set
            var reportTemplate = new TickHistoryTimeAndSalesReportTemplate
            {
                Name = "example-cancel" + Suffix.New(),
                Condition = new TickHistoryTimeAndSalesCondition
                {
                    QueryStartDate = new DateTimeOffset(new DateTime(2015, 12, 01, 00, 00, 00)),
                    QueryEndDate = new DateTimeOffset(new DateTime(2015, 12, 04, 23, 59, 59)),
                    MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                    ReportDateRangeType = ReportDateRangeType.Range,
                    ExtractBy = TickHistoryExtractByMode.Ric,
                    SortBy = TickHistorySort.SingleByRic,
                }
            };
            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "Quote - Ask Price"},
                new ContentField { FieldName = "Quote - Ask Size"},
                new ContentField { FieldName = "Quote - Ask Yield"},
                new ContentField { FieldName = "Quote - Bid Price"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create");
            //Define the schedule to be created.  The schedule is set to trigger immediately.
            var scheduleName = "example-eod" + Suffix.New();
            var schedule = new Schedule
            {
                Name = scheduleName,
                TimeZone = "Central Standard Time",
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow.AddDays(10), false),
                Trigger = ScheduleTrigger.CreateTimeTrigger(false, HourMinute.Create(17, 00)),
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "ExtractNow");
            //Force an extraction to occur so that we can demonstrate cancellation.  NOTE: ExtractNowStart is 
            //used for this example so that the request returns immediately while the request is processed.
            ExtractionsContext.ScheduleOperations.ExtractNowStart(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "PendingExtractions", MethodType.Association);
            //Retrieve the list of pending extractions.
            ExtractionsContext.LoadProperty(schedule, s => s.PendingExtractions);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportExtraction", "RequestCancellation", MethodType.Operation, Publish.Primary);
            //An attempt is made to cancel the specified extraction.  If the extraction does not support 
            //cancellation or is not in a state where it can be cancelled, a 400 / ValidationException will be 
            //returned.  If the method does not return an error, an attempt will be made to stop the extraction,
            //but it is possible the extraction will complete and usage/billing will be recorded as a successful 
            //transaction.  Cancellation is only supported for Tick History report templates.
            var reportExtraction = schedule.PendingExtractions
                .FirstOrDefault(re => re.ExtractionDateUtc < DateTime.UtcNow.AddDays(1)); //Ignore the pending extraction for 10 days from now
            try
            {
                if (reportExtraction != null)
                    ExtractionsContext.ReportExtractionOperations.RequestCancellation(schedule.PendingExtractions.First());
            }
            catch (ValidationException ex)
            {
                Status.WriteLine("Cancellation rejected: {0}", ex.Message);
            }
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportExtraction", "Get");
            //Get a fresh copy of the report extraction to see if the cancellation was accepted.
            if (reportExtraction != null)
            {
                reportExtraction = ExtractionsContext.ReportExtractionOperations.Get(reportExtraction.ReportExtractionId);
                Status.WriteLine("Report Extraction date: {0} status: {1}, detailed status: {2}",
                    reportExtraction.ExtractionDateUtc, reportExtraction.Status, reportExtraction.DetailedStatus);
            }
            Status.EndNotify(ExtractionsContext);

            //Delete the extracted files, schedule, instrument list and report template.  See the "Cleanup" example for
            //the various steps involved with cleanup.  NOTE: It is a good idea to clean up the extracted files (Report Extractions)
            //once you have downloaded the results.  If the schedule is not immediate (i.e. recurring), you want to keep the 
            //instrument list, report template and schedule around.
            Cleanup(schedule, instrumentList, reportTemplate);
        }

        #endregion

        #region Extract: Fixed Income Analytics

        [Example("Extract: Fixed Income Analytics")]
        public void FixedIncomeAnalytics()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.FixedIncomeAnalytics);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create the instrument list
            var instrumentList = new InstrumentList { Name = "ImmediateNonCustomized" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new[]
            {
                new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
                new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
            };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplates", "Create", MethodType.Operation, Publish.Primary);
            //Define a new end of day report template
            var reportTemplate = new FixedIncomeAnalyticsReportTemplate
            {
                Name = "ImmediateNonCustomized" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues
                //NOTE: There are many other fields which can be use to customize the report template
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "Instrument ID", "Instrument ID Type", "Asset Type", "Asset SubType", "Accrued Interest Rounding",
                "Bond Structure", "Calendar Adjustment Code", "Calendar Code", "Look Back Months", "Instrument Type",
                "Rate Structure", "Rate Type", "Bond Type", "Coupon Rate", "Coupon Rate Effective Date",
                "Currency Code", "CUSIP", "ISIN", "Maturity Date", "RIC", "Security Description", "SEDOL", "Ticker",
                "Usage Instrument SubType", "Usage Instrument Type", "User Defined Identifier", "Valoren", "Wertpapier",
                "Exception Code", "Exception Code Description");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedules", "Create");
            //Schedule the extraction to run immediately
            var schedule = new Schedule
            {
                Name = "ImmediateNonCustomized" + Suffix.New(),
                TimeZone = TimeZoneInfo.Local.Id,
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow, true),
                Trigger = ScheduleTrigger.CreateImmediateTrigger(true),
                ListId = instrumentList.ListId,
                ReportTemplateId = reportTemplate.ReportTemplateId,
            };
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            try
            {
                Status.Notify(ExtractionsContext, "ReportExtraction", "GetLastExtractionForScheduleId");
                //Wait for the extraction to complete.  This involves polling GetLastExtractionForScheduleId until 
                //the returned ReportExtraction.Status is completed.
                var extraction = ExtractionsContext.ScheduleOperations.WaitForNextExtraction(schedule);

                //Output some extraction information
                Status.WriteLine("status: {0} / detailedstatus: {1} / extraction date: {2} - {3} / files = {4}",
                    extraction.Status, extraction.DetailedStatus, extraction.ExtractionStartUtc, extraction.ExtractionEndUtc,
                    extraction.Files.Count);
                Status.EndNotify(ExtractionsContext);

                Status.Notify(ExtractionsContext, "ReportExtraction", "Files", MethodType.Association);
                //Get the list of files for this extraction.  This includes the contents, notes files and RIC maintenance files.
                ExtractionsContext.LoadProperty(extraction, "Files");

                //Find the extract file.  Other ExtractedFileTypes are: Note, RicMaintenanceNote, Full, Partial, Other
                var extractFile = extraction.Files.FirstOrDefault(f => f.FileType == ExtractedFileType.Full);
                Status.EndNotify(ExtractionsContext);

                if (extractFile != null)
                {
                    Status.Notify(ExtractionsContext, "ReportExtraction", "GetReadStream", MethodType.Stream, Publish.Secondary);
                    //Output the contents of the extracted file
                    var streamResponse = ExtractionsContext.ExtractedFileOperations.GetReadStream(extractFile);
                    string internalStreamOutput;
                    using (var reader = new StreamReader(streamResponse.Stream))
                    {
                        Status.WriteLine(internalStreamOutput = reader.ReadToEnd());
                    }
                    Status.EndNotify(ExtractionsContext, internalStreamOutput); // Extract the stream output and pass it as responseBody.
                }
            }
            catch (TimeoutException ex)
            {
                Status.WriteLine("Timed out: " + ex.Message);
            }

            //Delete the extracted files, schedule, instrument list and report template.  See the "Cleanup" example for
            //the various steps involved with cleanup.  NOTE: It is a good idea to clean up the extracted files (Report Extractions)
            //once you have downloaded the results.  If the schedule is not immediate (i.e. recurring), you want to keep the 
            //instrument list, report template and schedule around.
            Cleanup(schedule, instrumentList, reportTemplate);
        }

        #endregion

        #region Extract: End Of Day

        [Example("Extract: End Of Day")]
        public void EndOfDay()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EndOfDayPricing);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create the instrument list
            var instrumentList = new InstrumentList { Name = "ImmediateNonCustomized" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new[]
            {
                new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
                new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
            };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplates", "Create", MethodType.Operation, Publish.Secondary);
            //Define a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "ImmediateNonCustomized" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues
                //NOTE: There are many other fields which can be use to customize the report template
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "Ask Price", "Asset Category", "Asset Category Description", "Asset ID", "Asset Status",
                "Asset Status Description", "Asset SubType", "Asset SubType Description", "Asset Type",
                "Asset Type Description", "Bid Price", "Block Trades", "CIN Code", "Common Code",
                "Currency Code", "Currency Code Description", "Currency Code Scaled", "Currency Code Scaled Description",
                "CUSIP", "Exchange Code", "Exchange Description", "Exercise Style", "Expiration Date", "File Code",
                "High Price", "Instrument ID", "Instrument ID Type", "ISIN", "Issuer OrgID", "Lot Size", "Low Price",
                "Market MIC", "MIC", "Mid Price", "Net Asset Value", "Offer Price",
                "Official Close Price", "Open Price", "Previous Close Price", "Put Call Indicator", "Quote ID",
                "RCP ID", "RIC", "Security Description", "SEDOL", "Settlement Date", "Settlement Price", "SICC",
                "Sicovam", "Strike Price", "Ticker", "Trade Date", "Trading Status", "Trading Symbol", "Turnover",
                "Underlying RIC", "Universal Ask Price", "Universal Bid Ask Date", "Universal Bid Price",
                "Usage Instrument SubType", "Usage Instrument Type", "Valoren", "Volume", "VWAP Price", "Wertpapier");

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedules", "Create");
            //Schedule the extraction to run immediately
            var schedule = new Schedule
            {
                Name = "ImmediateNonCustomized" + Suffix.New(),
                TimeZone = TimeZoneInfo.Local.Id,
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow, true),
                Trigger = ScheduleTrigger.CreateImmediateTrigger(true),
                ListId = instrumentList.ListId,
                ReportTemplateId = reportTemplate.ReportTemplateId,
            };
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            try
            {
                Status.Notify(ExtractionsContext, "ReportExtraction", "GetLastExtractionForScheduleId");
                //Wait for the extraction to complete.  This involves polling GetLastExtractionForScheduleId until 
                //the returned ReportExtraction.Status is completed.
                var extraction = ExtractionsContext.ScheduleOperations.WaitForNextExtraction(schedule);

                //Output some extraction information
                Status.WriteLine("status: {0} / detailedstatus: {1} / extraction date: {2} - {3} / files = {4}",
                    extraction.Status, extraction.DetailedStatus, extraction.ExtractionStartUtc, extraction.ExtractionEndUtc,
                    extraction.Files.Count);
                Status.EndNotify(ExtractionsContext);

                Status.Notify(ExtractionsContext, "ReportExtraction", "Files", MethodType.Association);
                //Get the list of files for this extraction.  This includes the contents, notes files and RIC maintenance files.
                ExtractionsContext.LoadProperty(extraction, "Files");

                //Find the extract file.  Other ExtractedFileTypes are: Note, RicMaintenanceNote, Full, Partial, Other
                var extractFile = extraction.Files.FirstOrDefault(f => f.FileType == ExtractedFileType.Full);
                Status.EndNotify(ExtractionsContext);

                if (extractFile != null)
                {
                    Status.Notify(ExtractionsContext, "ReportExtraction", "GetReadStream", MethodType.Stream, Publish.Secondary);
                    //Output the contents of the extracted file
                    var streamResponse = ExtractionsContext.ExtractedFileOperations.GetReadStream(extractFile);
                    string internalStreamOutput;
                    using (var reader = new StreamReader(streamResponse.Stream))
                    {
                        Status.WriteLine(internalStreamOutput = reader.ReadToEnd());
                    }
                    Status.EndNotify(ExtractionsContext, internalStreamOutput); // Extract the stream output and pass it as responseBody.
                }
            }
            catch (TimeoutException ex)
            {
                Status.WriteLine("Timed out: " + ex.Message);
            }

            //Delete the extracted files, schedule, instrument list and report template.  See the "Cleanup" example for
            //the various steps involved with cleanup.  NOTE: It is a good idea to clean up the extracted files (Report Extractions)
            //once you have downloaded the results.  If the schedule is not immediate (i.e. recurring), you want to keep the 
            //instrument list, report template and schedule around.
            Cleanup(schedule, instrumentList, reportTemplate);
        }

        #endregion

        #region Extract: ISO 15022 (Corporate Actions)

        [Example("Extract: ISO 15022 (Corporate Actions)")]
        public void ExtractIso15022()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.CorporateActions);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create the instrument list
            var instrumentList = new InstrumentList { Name = "ImmediateNonCustomized" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new[]
            {
                new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
                new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
            };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplates", "Create", MethodType.Operation, Publish.Secondary);
            //Define a new end of day report template
            var reportTemplate = new CorporateActionsIsoReportTemplate
            {
                Name = Guid.NewGuid().ToString(),
                OutputFormat = ReportOutputFormat.IsoFormat,
                Condition = new CorporateActionsIsoCondition
                {
                    CorporateActionsStandardYearType = CorporateActionsStandardYearType.Current,
                    ExcludeNilPaidFromPaymentEvents = true,
                    GrossAmountOnlyForPaymentEvents = true,
                    IncludeNonPendingEvents = true,
                    PendingEventsHours = null,
                    PendingEventsMinutes = null,
                    PendingEventsPreviousDays = null,
                    PendingEventsPreviousMonths = null,
                    ReportDateRangeType = ReportDateRangeType.Init,
                    ReportIsoEvents = new[] { 
                        ReportIsoEventType.ACTV, ReportIsoEventType.BIDS, ReportIsoEventType.BONU, ReportIsoEventType.BPUT,
                        ReportIsoEventType.BRUP, ReportIsoEventType.CAPG, ReportIsoEventType.CAPI, ReportIsoEventType.CHAN, 
                        ReportIsoEventType.CMET, ReportIsoEventType.CONV, ReportIsoEventType.DECR, ReportIsoEventType.DFLT,
                        ReportIsoEventType.DLST, ReportIsoEventType.DRIP, ReportIsoEventType.DVCA, ReportIsoEventType.DVOP,
                        ReportIsoEventType.DVSE, ReportIsoEventType.EXOF, ReportIsoEventType.EXRI, ReportIsoEventType.EXTM,
                        ReportIsoEventType.EXWA, ReportIsoEventType.INCR, ReportIsoEventType.INTR, ReportIsoEventType.LIQU,
                        ReportIsoEventType.MCAL, ReportIsoEventType.MEET, ReportIsoEventType.MRGR, ReportIsoEventType.ODLT,
                        ReportIsoEventType.OMET, ReportIsoEventType.OTHR, ReportIsoEventType.PARI, ReportIsoEventType.PCAL,
                        ReportIsoEventType.PPMT, ReportIsoEventType.PRIO, ReportIsoEventType.REDM, ReportIsoEventType.REDO,
                        ReportIsoEventType.RHDI, ReportIsoEventType.RHTS, ReportIsoEventType.SHPR, ReportIsoEventType.SOFF,
                        ReportIsoEventType.SPLF, ReportIsoEventType.SPLR, ReportIsoEventType.SUSP, ReportIsoEventType.TEND,
                        ReportIsoEventType.XMET
                    }
                }
            };

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedules", "Create");
            //Schedule the extraction to run immediately
            var schedule = new Schedule
            {
                Name = "ImmediateNonCustomized" + Suffix.New(),
                TimeZone = TimeZoneInfo.Local.Id,
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow, true),
                Trigger = ScheduleTrigger.CreateImmediateTrigger(true),
                ListId = instrumentList.ListId,
                ReportTemplateId = reportTemplate.ReportTemplateId,
            };
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            try
            {
                Status.Notify(ExtractionsContext, "ReportExtraction", "GetLastExtractionForScheduleId");
                //Wait for the extraction to complete.  This involves polling GetLastExtractionForScheduleId until 
                //the returned ReportExtraction.Status is completed.
                var extraction = ExtractionsContext.ScheduleOperations.WaitForNextExtraction(schedule);

                //Output some extraction information
                Status.WriteLine("status: {0} / detailedstatus: {1} / extraction date: {2} - {3} / files = {4}",
                    extraction.Status, extraction.DetailedStatus, extraction.ExtractionStartUtc, extraction.ExtractionEndUtc,
                    extraction.Files.Count);
                Status.EndNotify(ExtractionsContext);

                Status.Notify(ExtractionsContext, "ReportExtraction", "Files", MethodType.Association);
                //Get the list of files for this extraction.  This includes the contents, notes files and RIC maintenance files.
                ExtractionsContext.LoadProperty(extraction, "Files");

                //Find the extract file.  Other ExtractedFileTypes are: Note, RicMaintenanceNote, Full, Partial, Other
                var extractFile = extraction.Files.FirstOrDefault(f => f.FileType == ExtractedFileType.Full);
                Status.EndNotify(ExtractionsContext);

                if (extractFile != null)
                {
                    Status.Notify(ExtractionsContext, "ReportExtraction", "GetReadStream", MethodType.Stream, Publish.Secondary);
                    //Output the contents of the extracted file
                    var streamResponse = ExtractionsContext.ExtractedFileOperations.GetReadStream(extractFile);
                    string internalStreamOutput;
                    using (var reader = new StreamReader(streamResponse.Stream))
                    {
                        Status.WriteLine(internalStreamOutput = reader.ReadToEnd());
                    }
                    Status.EndNotify(ExtractionsContext, internalStreamOutput); // Extract the stream output and pass it as responseBody.
                }
            }
            catch (TimeoutException ex)
            {
                Status.WriteLine("Timed out: " + ex.Message);
            }

            //Delete the extracted files, schedule, instrument list and report template.  See the "Cleanup" example for
            //the various steps involved with cleanup.  NOTE: It is a good idea to clean up the extracted files (Report Extractions)
            //once you have downloaded the results.  If the schedule is not immediate (i.e. recurring), you want to keep the 
            //instrument list, report template and schedule around.
            Cleanup(schedule, instrumentList, reportTemplate);
        }

        #endregion

        #region Extract: ISO 15022 (Debt Corporate Actions)

        [Example("Extract: ISO 15022 (Debt Corporate Actions)")]
        public void ExtractDebtIso15022()
        {
            Status.Notify(ExtractionsContext, "ReportTemplateType", "GetContentFieldTypes", MethodType.Operation, Publish.Secondary);
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = ExtractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.DebtCorporateActions);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create the instrument list
            var instrumentList = new InstrumentList { Name = "ImmediateNonCustomized" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new[]
            {
                new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
                new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
            };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportTemplates", "Create", MethodType.Operation, Publish.Secondary);
            //Define a new end of day report template
            var reportTemplate = new DebtCorporateActionsIsoReportTemplate
            {
                Name = Guid.NewGuid().ToString(),
                OutputFormat = ReportOutputFormat.IsoFormat,
                Condition = new DebtCorporateActionsIsoCondition
                {
                    DebtCorporateActionsStandardYearType = DebtCorporateActionsStandardYearType.Current,
                    IncludeNonPendingEvents = true,
                    PendingEventsHours = null,
                    PendingEventsMinutes = null,
                    PendingEventsPreviousDays = null,
                    PendingEventsPreviousMonths = null,
                    ReportDateRangeType = ReportDateRangeType.Init,
                    ReportIsoEvents = new[] { ReportDebtIsoEventType.BRUP }
                }
            };

            //Create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedules", "Create");
            //Schedule the extraction to run immediately
            var schedule = new Schedule
            {
                Name = "ImmediateNonCustomized" + Suffix.New(),
                TimeZone = TimeZoneInfo.Local.Id,
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow, true),
                Trigger = ScheduleTrigger.CreateImmediateTrigger(true),
                ListId = instrumentList.ListId,
                ReportTemplateId = reportTemplate.ReportTemplateId,
            };
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            try
            {
                Status.Notify(ExtractionsContext, "ReportExtraction", "GetLastExtractionForScheduleId");
                //Wait for the extraction to complete.  This involves polling GetLastExtractionForScheduleId until 
                //the returned ReportExtraction.Status is completed.
                var extraction = ExtractionsContext.ScheduleOperations.WaitForNextExtraction(schedule);

                //Output some extraction information
                Status.WriteLine("status: {0} / detailedstatus: {1} / extraction date: {2} - {3} / files = {4}",
                    extraction.Status, extraction.DetailedStatus, extraction.ExtractionStartUtc, extraction.ExtractionEndUtc,
                    extraction.Files.Count);
                Status.EndNotify(ExtractionsContext);

                Status.Notify(ExtractionsContext, "ReportExtraction", "Files", MethodType.Association);
                //Get the list of files for this extraction.  This includes the contents, notes files and RIC maintenance files.
                ExtractionsContext.LoadProperty(extraction, "Files");

                //Find the extract file.  Other ExtractedFileTypes are: Note, RicMaintenanceNote, Full, Partial, Other
                var extractFile = extraction.Files.FirstOrDefault(f => f.FileType == ExtractedFileType.Full);
                Status.EndNotify(ExtractionsContext);

                if (extractFile != null)
                {
                    Status.Notify(ExtractionsContext, "ReportExtraction", "GetReadStream", MethodType.Stream, Publish.Secondary);
                    //Output the contents of the extracted file
                    var streamResponse = ExtractionsContext.ExtractedFileOperations.GetReadStream(extractFile);
                    string internalStreamOutput;
                    using (var reader = new StreamReader(streamResponse.Stream))
                    {
                        Status.WriteLine(internalStreamOutput = reader.ReadToEnd());
                    }
                    Status.EndNotify(ExtractionsContext, internalStreamOutput); // Extract the stream output and pass it as responseBody.
                }
            }
            catch (TimeoutException ex)
            {
                Status.WriteLine("Timed out: " + ex.Message);
            }

            //Delete the extracted files, schedule, instrument list and report template.  See the "Cleanup" example for
            //the various steps involved with cleanup.  NOTE: It is a good idea to clean up the extracted files (Report Extractions)
            //once you have downloaded the results.  If the schedule is not immediate (i.e. recurring), you want to keep the 
            //instrument list, report template and schedule around.
            Cleanup(schedule, instrumentList, reportTemplate);
        }

        #endregion

        #region Extracted Files

        [Example("Get Latest")]
        public void GetLatest()
        {
            /* This example provides an alternate mechanism for retrieving the extraction contents for a schedule or schedules.
             * The preferred approach is to use the WaitForNextExtraction method on the toolkit (or LastExtraction association 
             * property). The MaxPageSize defaults to 250 and should be kept at the default.  If the first page's results have 
             * the HasNextPage set to true then more data is available.  You can continue to retrieve subsequent pages using
             * GetAll and passing in the next link.  When no more data is available, HasNextPage will be false. */
            ExtractionsContext.Preferences.MaxPageSize = 3; //Only set for the purposes of the example

            Status.Notify(ExtractionsContext, "ReportExtraction", "GetAll", MethodType.Operation, Publish.Primary);
            //Returns the first page (250) of the latest Report Extractions for all Schedules.
            var firstPage = ExtractionsContext.ReportExtractionOperations.GetAll();

            //Output
            //Use the schedule id to determine which schedule the results are associated to.
            Status.WriteLine("FIRST PAGE:");
            foreach (var reportExtraction in firstPage)
                Status.WriteLine("ScheduleName: {0}, Date: {1}",
                    reportExtraction.ScheduleName, reportExtraction.ExtractionDateUtc);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportExtraction", "GetAll", MethodType.Operation, Publish.Secondary);
            //Only get the second page of data if data is available.
            if (firstPage.HasNextPage)
            {
                //Returns the second page.  Notice that NextLink is passed to GetAll.
                var secondPage = ExtractionsContext.ReportExtractionOperations.GetAll(firstPage.NextLink);

                //Output
                Status.WriteLine("SECOND PAGE:");
                foreach (var reportExtraction in secondPage)
                    Status.WriteLine("ScheduleName: {0}, Date: {1}",
                        reportExtraction.ScheduleName, reportExtraction.ExtractionDateUtc);
            }
            Status.EndNotify(ExtractionsContext);

        }

        #endregion

        #region Extracted Files

        [Example("ExtractedFiles: Get Latest")]
        public void ExtractedFilesGetLatest()
        {
            /* This example provides an alternate mechanism for retrieving the extraction contents for a schedule or schedules.
             * The preferred approach is to use the WaitForNextExtraction method on the toolkit (or LastExtraction association 
             * property). The MaxPageSize defaults to 250 and should be kept at the default.  If the first page's results have 
             * the HasNextPage set to true then more data is available.  You can continue to retrieve subsequent pages using
             * GetAll and passing in the next link.  When no more data is available, HasNextPage will be false. */
            ExtractionsContext.Preferences.MaxPageSize = 3; //Only set for the purposes of the example

            Status.Notify(ExtractionsContext, "ExtractedFile", "GetAll", MethodType.Operation, Publish.Primary);
            //Returns the first page (250) of the latest Extracted Files for all Report Extractions for all Schedules.
            var firstPage = ExtractionsContext.ExtractedFileOperations.GetAll();

            //Output
            //Use the schedule id to determine which schedule the results are associated to.
            Status.WriteLine("FIRST PAGE:");
            foreach (var extractedFile in firstPage)
                Status.WriteLine("ScheduleId: {0}, Filename: {1}, Size: {2}, ContentsExist: {3}",
                    extractedFile.ScheduleId, extractedFile.ExtractedFileName, extractedFile.Size, extractedFile.ContentsExists);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ExtractedFile", "GetAll", MethodType.Operation, Publish.Secondary);
            //Only get the second page of data if data is available.
            if (firstPage.HasNextPage)
            {
                //Returns the second page of Extracted Files.  Notice that NextLink is passed to GetAll.
                var secondPage = ExtractionsContext.ExtractedFileOperations.GetAll(firstPage.NextLink);

                //Output
                //Use the schedule id to determine which schedule the results are associated to.
                Status.WriteLine("SECOND PAGE:");
                foreach (var extractedFile in secondPage)
                    Status.WriteLine("ScheduleId: {0}, Filename: {1}, Size: {2}, ContentsExist: {3}",
                        extractedFile.ScheduleId, extractedFile.ExtractedFileName, extractedFile.Size, extractedFile.ContentsExists);
            }
            Status.EndNotify(ExtractionsContext);

        }

        [Example("ExtractedFiles: Get Changes")]
        public void ExtractedFilesGetChanges()
        {
            /* This example provides an alternate mechanism for retrieving the extraction contents for a schedule or schedules.
             * The preferred approach is to use the WaitForNextExtraction method on the toolkit (or LastExtraction association 
             * property).
             * If Change Tracking is enabled the server returns a delta token along with the payload.  If the delta token is 
             * presented on a subsequent request only records that have changed since the prior request are returned.
             * NOTE: Paging is in effect along with change tracking. */
            ExtractionsContext.Preferences.MaxPageSize = 3; //Only set for the purposes of the example
            ExtractionsContext.Preferences.TrackChanges = true; //Let the API know that you are interested in tracking changes.

            Status.Notify(ExtractionsContext, "ExtractedFile", "GetAll", MethodType.Operation, Publish.Secondary);
            //Get all the results.  The results are paged so that a maximum of 250 are returned.  Because TrackChanges was set to
            //true, firstPage.DeltaLink includes a token that can be used to find any changes that were made since this request.
            //If you _really_ wanted all the results here you would have to call GetAll with all.NextLink until HasNextPage is 
            //false.
            var all = ExtractionsContext.ExtractedFileOperations.GetAll();

            //Output
            Status.WriteLine("ALL:");
            //NOTE: DeltaLink's can be persisted using DeltaLink.ToString() and DeltaLink.Parse().
            Status.WriteLine("DeltaLink: " + all.DeltaLink.Url);
            foreach (var extractedFile in all)
                Status.WriteLine("ScheduleId: {0}, Filename: {1}, Size: {2}, ContentsExist: {3}",
                    extractedFile.ScheduleId, extractedFile.ExtractedFileName, extractedFile.Size, extractedFile.ContentsExists);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ExtractedFile", "GetAll", MethodType.Operation, Publish.Secondary);
            //Get all the ExtractedFiles that have changed since the original request.  Note that these results can also be paged
            //which means that you would need to call GetAll with changed.NextLink while changed.HasNextPage is true.
            var changed = ExtractionsContext.ExtractedFileOperations.GetAll(all.DeltaLink);

            //Output
            Status.WriteLine("CHANGES:");
            //In this example there will be no changes since no extractions were likely to have run between these two calls to
            //GetAll.
            foreach (var extractedFile in changed)
                Status.WriteLine("ScheduleId: {0}, Filename: {1}, Size: {2}, ContentsExist: {3}",
                    extractedFile.ScheduleId, extractedFile.ExtractedFileName, extractedFile.Size, extractedFile.ContentsExists);
            Status.EndNotify(ExtractionsContext);

        }

        #endregion

        #region Cleanup Helper Method

        // This cleanup code serves as an example of how to do a get-all for the various entity types.
        // The cleanup button is useful if you abort debug sessions while exploring these samples.  
        // Cleanup is by name prefix.  It removes all lists, reports, schedules, and file content that matches the prefix.

        public void Cleanup(Schedule schedule, InstrumentList instrumentList, ReportTemplate reportTemplate)
        {
            Cleanup(schedule, reportTemplate);

            //Delete the instrument list
            if (instrumentList != null && instrumentList.ListId != null)
                ExtractionsContext.InstrumentListOperations.Delete(instrumentList);
        }

        public void Cleanup(Schedule schedule, EntityList entityList, ReportTemplate reportTemplate)
        {
            Cleanup(schedule, reportTemplate);

            //Delete the entity list
            if (entityList != null && entityList.ListId != null)
                ExtractionsContext.EntityListOperations.Delete(entityList);
        }

        public void Cleanup(Schedule schedule, ReportTemplate reportTemplate)
        {
            // The order of cleanup is intentional.  Minimizes the potential impact of orphaned entities (if any).

            // ReportExtractions and ExtractedFiles are immutable, but the file content needs to be cleaned up.
            // The ReportExtractions and ExtractedFiles will still be retrievable, but the file content will be gone.

            if (schedule != null && schedule.ScheduleId != null)
            {
                var scheduleExtractions = ExtractionsContext.ReportExtractionOperations.GetByScheduleId(schedule.ScheduleId);
                foreach (var reportExtraction in scheduleExtractions)
                {
                    ExtractionsContext.LoadProperty(reportExtraction, "Files");
                    var fileIds = reportExtraction.Files
                                                  .Where(f => f.ContentsExists)
                                                  .Select(f => f.ExtractedFileId)
                                                  .ToList();
                    if (fileIds.Count > 0)
                        ExtractionsContext.ExtractedFileOperations.DeleteMultiple(fileIds);
                }
                ExtractionsContext.ScheduleOperations.Delete(schedule);
            }

            if (reportTemplate != null && reportTemplate.ReportTemplateId != null)
                ExtractionsContext.ReportTemplateOperations.Delete(reportTemplate);
        }

        #endregion
    }

}
