﻿using Dss.Api.Examples.Internal;
using System;
using System.Collections.Generic;
using System.Text;
using DataScope.Select.Api;
using DataScope.Select.Api.Content;
using DataScope.Select.Api.Extractions;
using DataScope.Select.Api.Extractions.ReportTemplates;
using DataScope.Select.Api.Extractions.Schedules;
using DataScope.Select.Api.Extractions.SubjectLists;

namespace Dss.Api.Examples.Extractions.Schedules
{

    [Example("Schedule Examples")]
    public class ScheduleExamples
    {
        protected Uri ServiceLayerUri { get; set; }
        protected ExtractionsContext ExtractionsContext { get; set; }
        public const string NamePrefix = "_eP-";

        protected Schedule Schedule { get; set; }
        protected InstrumentList InstrumentList { get; set; }
        protected EntityList EntityList { get; set; }
        protected EndOfDayPricingReportTemplate EodReportTemplate { get; set; }
        protected LegalEntityAuditReportTemplate AuditReportTemplate { get; set; }

        #region Setup and Cleanup

        public ScheduleExamples()
        {
            ExtractionsContext = ContextHelper.CreateExtractionsContext();
        }

        private void CleanUp()
        {
            if (Schedule != null)
            {
                ExtractionsContext.ScheduleOperations.Delete(Schedule);
            }
            Schedule = null;

            if (InstrumentList != null)
            {
                ExtractionsContext.InstrumentListOperations.Delete(InstrumentList);
            }
            InstrumentList = null;

            if (EntityList != null)
            {
                ExtractionsContext.EntityListOperations.Delete(EntityList);
            }
            EntityList = null;

            if (EodReportTemplate != null)
            {
                ExtractionsContext.ReportTemplateOperations.Delete(EodReportTemplate);
            }
            EodReportTemplate = null;

            if (AuditReportTemplate != null)
            {
                ExtractionsContext.ReportTemplateOperations.Delete(AuditReportTemplate);
            }
            AuditReportTemplate = null;
        }

        #endregion

        #region Query

        /// <summary>
        /// This example shows how to get all the schedules for the logged in user.
        /// </summary>
        [Example("Get All (Paged Results)")]
        public void GetAllSchedules()
        {
            //Set the page size to 1 record per page for the purposes of this example
            ExtractionsContext.Preferences.MaxPageSize = 1;

            Status.Notify(ExtractionsContext, "Schedule", "GetAll", MethodType.Operation, Publish.Primary);
            //Fetch the first page (default page size is 250)
            var page1Results = ExtractionsContext.ScheduleOperations.GetAll();

            //Output the list of schedule names
            foreach (var schedule in page1Results)
                Status.WriteLine(schedule.Name);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "GetAll", MethodType.Operation, Publish.Secondary);
            if (page1Results.HasNextPage) //Use a while loop if you need to retrieve all instrument lists
            {
                //Fetch the second page of schedules by passing in the NextLink.
                var page2Results = ExtractionsContext.ScheduleOperations.GetAll(page1Results.NextLink);

                //Output the list of schedule names
                foreach (var schedule in page2Results)
                    Status.WriteLine(schedule.Name);
            }
            Status.EndNotify(ExtractionsContext);
        }

        /// <summary>
        /// This example shows how to retrieve a schedule by its name.
        /// </summary>
        [Example("Get: By Name")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(Schedule))]
        public void GetScheduleByName()
        {
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "example-eod" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
			    {
				    new InstrumentIdentifier
					    {
						    Identifier = "IBM.N",
						    IdentifierType = IdentifierType.Ric,
						    UserDefinedIdentifier = "UserIdent1A", //User identifiers are optional
						    UserDefinedIdentifier2 = "UserIdent1B",
						    UserDefinedIdentifier3 = "UserIdent1C"
					    },
				    new InstrumentIdentifier
					    {
						    Identifier = "KO.N",
						    IdentifierType = IdentifierType.Ric,
					    }
			    };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EndOfDayPricingReportTemplate", "Create");
            //Define the report template specific to the desired content set
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "example-eod" + Suffix.New(),
            };

            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "52 Week High"},
                new ContentField { FieldName = "7 Day Compound Yield"},
                new ContentField { FieldName = "Ask Price"},
                new ContentField { FieldName = "Close Price"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create");
            //Define the schedule to be created
            var scheduleName = "example-eod" + Suffix.New();
            var newSchedule = new Schedule
            {
                Name = scheduleName,
                TimeZone = "Central Standard Time",
                Recurrence = ScheduleRecurrence.CreateWeeklyRecurrence(
                    new[] { DayOfWeek.Monday, DayOfWeek.Friday }),
                Trigger = ScheduleTrigger.CreateTimeTrigger(
                    false, //Limit to todays data
                    new[] { new HourMinute { Hour = 16, Minute = 0 } }),
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(newSchedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "GetByName", MethodType.Operation, Publish.Primary);
            //Fetch the schedule by name
            var schedule = ExtractionsContext.ScheduleOperations.GetByName(scheduleName);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Create

        /// <summary>
        /// This example shows how to create an End of Day Schedule.  It will create an 
        /// instrument list and report template that will be used by the schedule.
        /// It creates a new schedule that is time triggered on a weekly recurrence.
        /// </summary>
        /// <returns>Created Schedule</returns>
        [Example("Create: for End of Day")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(Schedule))]
        public void CreateEodSchedule()
        {
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "example-eod" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
			    {
				    new InstrumentIdentifier
					    {
						    Identifier = "IBM.N",
						    IdentifierType = IdentifierType.Ric,
						    UserDefinedIdentifier = "UserIdent1A", //User identifiers are optional
						    UserDefinedIdentifier2 = "UserIdent1B",
						    UserDefinedIdentifier3 = "UserIdent1C"
					    },
				    new InstrumentIdentifier
					    {
						    Identifier = "KO.N",
						    IdentifierType = IdentifierType.Ric,
					    }
			    };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EndOfDayPricingReportTemplate", "Create");
            //Define the report template specific to the desired content set
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "example-eod" + Suffix.New(),
            };

            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "52 Week High"},
                new ContentField { FieldName = "7 Day Compound Yield"},
                new ContentField { FieldName = "Ask Price"},
                new ContentField { FieldName = "Close Price"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create", MethodType.Operation, Publish.Primary);
            //Define the schedule to be created
            var schedule = new Schedule
            {
                Name = "example-eod" + Suffix.New(),
                TimeZone = "Central Standard Time",
                Recurrence = ScheduleRecurrence.CreateWeeklyRecurrence(
                    new[] { DayOfWeek.Monday, DayOfWeek.Friday }),
                Trigger = ScheduleTrigger.CreateTimeTrigger(
                    false, //Limit to todays data
                    new[] { new HourMinute { Hour = 16, Minute = 0 } }),
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);
        }

        /// <summary>
        /// This example shows how to create an Legal Entity Audit Schedule.  It will create an 
        /// instrument list and report template that will be used by the schedule.
        /// It creates a new schedule that is time triggered on a weekly recurrence.
        /// </summary>
        /// <returns>Created Schedule</returns>
        [Example("Create: for Legal Entity Audit (Entity List)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(Schedule))]
        public void CreateLegalEntityAuditSchedule()
        {
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "EntityList", "Create");
            //Create a new instrument list
            var entityList = new EntityList { Name = "example-entity" + Suffix.New() };
            ExtractionsContext.EntityListOperations.Create(entityList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EntityList", "AppendIdentifiers");
            //Add instruments to the list
            var entities = new List<EntityIdentifier>
			    {
				    new EntityIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
				    new EntityIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
			    };
            ExtractionsContext.EntityListOperations.AppendIdentifiers(entityList, entities, false, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "LegalEntityAuditReportTemplate", "Create");
            //Define the report template for the desired content
            var reportTemplate = new LegalEntityAuditReportTemplate
            {
                Name = "example-entity" + Suffix.New(),
            };

            //Specify the list of interested fields for this report template / content set
            reportTemplate.ContentFields.AddRange(
                "Audit Modified Date", "Audit Source", "Audit Source Reference", "Audited Field Name", "ID", "ID Type",
                "Official Name", "RCP ID", "User Defined Identifier");

            //Post the request to create the report template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create", MethodType.Operation, Publish.Secondary);
            //Define the schedule to be created
            var schedule = new Schedule
            {
                Name = "example-entity" + Suffix.New(),
                TimeZone = TimeZoneInfo.GetSystemTimeZones()[1].Id,
                Recurrence = ScheduleRecurrence.CreateWeeklyRecurrence(
                    new[] { DayOfWeek.Monday, DayOfWeek.Friday }),
                Trigger = ScheduleTrigger.CreateTimeTrigger(
                    false, //Limit to todays data
                    new[] { new HourMinute { Hour = 16, Minute = 0 } }),
                ListId = entityList.ListId,
                ReportTemplateId = reportTemplate.ReportTemplateId,
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Weekly Recurring")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(Schedule))]
        public void CreateWeeklyRecurringSchedule()
        {
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "example-eod" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
			    {
				    new InstrumentIdentifier
					    {
						    Identifier = "IBM.N",
						    IdentifierType = IdentifierType.Ric,
						    UserDefinedIdentifier = "UserIdent1A", //User identifiers are optional
						    UserDefinedIdentifier2 = "UserIdent1B",
						    UserDefinedIdentifier3 = "UserIdent1C"
					    },
				    new InstrumentIdentifier
					    {
						    Identifier = "KO.N",
						    IdentifierType = IdentifierType.Ric,
					    }
			    };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EndOfDayPricingReportTemplate", "Create");
            //Define the report template specific to the desired content set
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "example-eod" + Suffix.New(),
            };

            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "52 Week High"},
                new ContentField { FieldName = "7 Day Compound Yield"},
                new ContentField { FieldName = "Ask Price"},
                new ContentField { FieldName = "Close Price"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create", MethodType.Operation, Publish.Secondary);
            //Define the schedule to be created
            var schedule = new Schedule
            {
                Name = "example-eod" + Suffix.New(),
                TimeZone = "Central Standard Time",
                //Extraction will run every week on Monday and Friday
                Recurrence = ScheduleRecurrence.CreateWeeklyRecurrence(
                    new[] { DayOfWeek.Monday, DayOfWeek.Friday }),
                //Extraction will run at 4pm in Central Standard Time zone
                Trigger = ScheduleTrigger.CreateTimeTrigger(
                    false, //Limit to todays data
                    new[] { new HourMinute { Hour = 16, Minute = 0 } }),
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Monthly Recurring")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(Schedule))]
        public void CreateMonthlyRecurringSchedule()
        {
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "example-eod" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
			    {
				    new InstrumentIdentifier
					    {
						    Identifier = "IBM.N",
						    IdentifierType = IdentifierType.Ric,
						    UserDefinedIdentifier = "UserIdent1A", //User identifiers are optional
						    UserDefinedIdentifier2 = "UserIdent1B",
						    UserDefinedIdentifier3 = "UserIdent1C"
					    },
				    new InstrumentIdentifier
					    {
						    Identifier = "KO.N",
						    IdentifierType = IdentifierType.Ric,
					    }
			    };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EndOfDayPricingReportTemplate", "Create");
            //Define the report template specific to the desired content set
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "example-eod" + Suffix.New(),
            };

            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "52 Week High"},
                new ContentField { FieldName = "7 Day Compound Yield"},
                new ContentField { FieldName = "Ask Price"},
                new ContentField { FieldName = "Close Price"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create", MethodType.Operation, Publish.Secondary);
            //Define the schedule to be created
            var schedule = new Schedule
            {
                Name = "example-eod" + Suffix.New(),
                TimeZone = "Central Standard Time",
                //Extraction will occur on the first of every month
                Recurrence = ScheduleRecurrence.CreateMonthlyRecurrence(1),
                //Extraction will run at 4pm in Central Standard Time zone
                Trigger = ScheduleTrigger.CreateTimeTrigger(
                    false, //Limit to todays data
                    new[] { new HourMinute { Hour = 16, Minute = 0 } }),
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: One Time")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(Schedule))]
        public void CreateOneTimeSchedule()
        {
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "example-eod" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
			    {
				    new InstrumentIdentifier
					    {
						    Identifier = "IBM.N",
						    IdentifierType = IdentifierType.Ric,
						    UserDefinedIdentifier = "UserIdent1A", //User identifiers are optional
						    UserDefinedIdentifier2 = "UserIdent1B",
						    UserDefinedIdentifier3 = "UserIdent1C"
					    },
				    new InstrumentIdentifier
					    {
						    Identifier = "KO.N",
						    IdentifierType = IdentifierType.Ric,
					    }
			    };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EndOfDayPricingReportTemplate", "Create");
            //Define the report template specific to the desired content set
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "example-eod" + Suffix.New(),
            };

            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "52 Week High"},
                new ContentField { FieldName = "7 Day Compound Yield"},
                new ContentField { FieldName = "Ask Price"},
                new ContentField { FieldName = "Close Price"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create", MethodType.Operation, Publish.Secondary);
            //Define the schedule to be created
            var schedule = new Schedule
            {
                Name = "example-eod" + Suffix.New(),
                TimeZone = "Central Standard Time",
                //Specify the date to run (tomorrow)
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTime.UtcNow.AddDays(1).Date, false /* Is Immediate */),
                //Specify the time of day
                Trigger = ScheduleTrigger.CreateTimeTrigger(
                    false, //Limit to todays data
                    new[] { new HourMinute { Hour = 16, Minute = 0 } }),
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Data Availability Triggered")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(Schedule))]
        public void CreateDataAvailabilityTriggered()
        {
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "example-eod" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
			    {
				    new InstrumentIdentifier
					    {
						    Identifier = "IBM.N",
						    IdentifierType = IdentifierType.Ric,
						    UserDefinedIdentifier = "UserIdent1A", //User identifiers are optional
						    UserDefinedIdentifier2 = "UserIdent1B",
						    UserDefinedIdentifier3 = "UserIdent1C"
					    },
				    new InstrumentIdentifier
					    {
						    Identifier = "KO.N",
						    IdentifierType = IdentifierType.Ric,
					    }
			    };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EndOfDayPricingReportTemplate", "Create");
            //Define the report template specific to the desired content set
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "example-eod" + Suffix.New(),
            };

            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "52 Week High"},
                new ContentField { FieldName = "7 Day Compound Yield"},
                new ContentField { FieldName = "Ask Price"},
                new ContentField { FieldName = "Close Price"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create", MethodType.Operation, Publish.Secondary);
            //Define the schedule to be created
            var schedule = new Schedule
            {
                Name = "example-eod" + Suffix.New(),
                TimeZone = "Central Standard Time",
                //Specify the date to run (tomorrow)
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTime.UtcNow.AddDays(1).Date, false /* Is Immediate */),
                //Wait for the latest end of day data before running the extraction
                Trigger = ScheduleTrigger.CreateDataAvailabilityTrigger(
                    true, //Limit to Todays Data
                    ExtendedTriggerInformation.CreateEndOfDay(true /* Error Corrected */, false /* Delta */),
                    DataAvailabilityTriggerTimeout.Create(
                        HourMinute.Create(2, 0),    //Time Out - how long to wait before terminating the request
                        0)),                        //TriggerTimeoutDayOffset - 
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Composite Triggered")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(Schedule))]
        public void CreateCompositeTriggered()
        {
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "example-composite" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
			    {
				    new InstrumentIdentifier
					    {
						    Identifier = "IBM.N",
						    IdentifierType = IdentifierType.Ric,
						    UserDefinedIdentifier = "UserIdent1A", //User identifiers are optional
						    UserDefinedIdentifier2 = "UserIdent1B",
						    UserDefinedIdentifier3 = "UserIdent1C"
					    },
				    new InstrumentIdentifier
					    {
						    Identifier = "KO.N",
						    IdentifierType = IdentifierType.Ric,
					    }
			    };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "PremiumEndOfDayPricingReportTemplate", "Create");
            //Define the report template specific to the desired content set
            var reportTemplate = new CompositeReportTemplate()
            {
                Name = "example-composite" + Suffix.New(),
            };

            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "52 Week High"},
                new ContentField { FieldName = "7 Day Compound Yield"},
                new ContentField { FieldName = "Ask Price"},
                new ContentField { FieldName = "Close Price"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create", MethodType.Operation, Publish.Secondary);
            //Define the schedule to be created
            var schedule = new Schedule
            {
                Name = "example-composite" + Suffix.New(),
                TimeZone = "Central Standard Time",
                //Specify the date to run (tomorrow)
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTime.UtcNow.AddDays(1).Date, false /* Is Immediate */),
                //Wait for the latest end of day data before running the extraction
                Trigger = ScheduleTrigger.CreateDataAvailabilityTrigger(
                    true, //Limit to Todays Data
                    ExtendedTriggerInformation.CreateComposite(true /* Error Corrected */, false /* Delta */),
                    DataAvailabilityTriggerTimeout.Create(
                        HourMinute.Create(2, 0),    //Time Out - how long to wait before terminating the request
                        0)),                        //TriggerTimeoutDayOffset - 
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);
        }

        [Example("Create: Elektron End of Day Pricing - Data Availability Triggered")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(Schedule))]
        public void CreateElektronEndOfDayPricingDataAvailabilityTriggered()
        {
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "example-eed" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
                {
                    new InstrumentIdentifier
                        {
                            Identifier = "IBM.N",
                            IdentifierType = IdentifierType.Ric,
                            UserDefinedIdentifier = "UserIdent1A", //User identifiers are optional
						    UserDefinedIdentifier2 = "UserIdent1B",
                            UserDefinedIdentifier3 = "UserIdent1C"
                        },
                    new InstrumentIdentifier
                        {
                            Identifier = "KO.N",
                            IdentifierType = IdentifierType.Ric,
                        }
                };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ElektronEndOfDayPricingReportTemplate", "Create");
            //Define the report template specific to the desired content set
            var reportTemplate = new ElektronEndOfDayPricingReportTemplate
            {
                Name = "example-eed" + Suffix.New(),
            };

            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "Ask Price"},
                new ContentField { FieldName = "Bid Price"},
                new ContentField { FieldName = "Fair Value"},
                new ContentField { FieldName = "Lot Size"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create", MethodType.Operation, Publish.Secondary);
            //Define the schedule to be created
            var schedule = new Schedule
            {
                Name = "example-eed" + Suffix.New(),
                TimeZone = "Central Standard Time",
                //Specify the date to run (tomorrow)
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTime.UtcNow.AddDays(1).Date, false /* Is Immediate */),
                //Wait for the latest end of day data before running the extraction
                Trigger = ScheduleTrigger.CreateDataAvailabilityTrigger(
                    true, //Limit to Todays Data
                    ExtendedTriggerInformation.CreateElektronEndOfDayPricing(true /* Error Corrected */),
                    DataAvailabilityTriggerTimeout.Create(
                        HourMinute.Create(2, 0),    //Time Out - how long to wait before terminating the request
                        0)),                        //TriggerTimeoutDayOffset - 
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);
        }


        [Example("Create: Immediate")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(Schedule))]
        public void CreateImmediate()
        {
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "example-eod" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
			    {
				    new InstrumentIdentifier
					    {
						    Identifier = "IBM.N",
						    IdentifierType = IdentifierType.Ric,
						    UserDefinedIdentifier = "UserIdent1A", //User identifiers are optional
						    UserDefinedIdentifier2 = "UserIdent1B",
						    UserDefinedIdentifier3 = "UserIdent1C"
					    },
				    new InstrumentIdentifier
					    {
						    Identifier = "KO.N",
						    IdentifierType = IdentifierType.Ric,
					    }
			    };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EndOfDayPricingReportTemplate", "Create");
            //Define the report template specific to the desired content set
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "example-eod" + Suffix.New(),
            };

            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "52 Week High"},
                new ContentField { FieldName = "7 Day Compound Yield"},
                new ContentField { FieldName = "Ask Price"},
                new ContentField { FieldName = "Close Price"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create", MethodType.Operation, Publish.Secondary);
            //Define the schedule to be created
            var schedule = new Schedule
            {
                Name = "example-eod" + Suffix.New(),
                TimeZone = "Central Standard Time",
                //Specify the date to run (tomorrow)
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTime.Now, true /* Immediate */),
                //Specify to run the extraction immediately
                Trigger = ScheduleTrigger.CreateImmediateTrigger(true /* Limit to Todays data */),
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Update

        /// <summary>
        /// This example shows how to update a previously create schedule's trigger and recurrence.
        /// </summary>
        [Example("Update")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(Schedule))]
        public void UpdateScheduleTriggerAndRecurrence()
        {
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "example-eod" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
			    {
				    new InstrumentIdentifier
					    {
						    Identifier = "IBM.N",
						    IdentifierType = IdentifierType.Ric,
						    UserDefinedIdentifier = "UserIdent1A", //User identifiers are optional
						    UserDefinedIdentifier2 = "UserIdent1B",
						    UserDefinedIdentifier3 = "UserIdent1C"
					    },
				    new InstrumentIdentifier
					    {
						    Identifier = "KO.N",
						    IdentifierType = IdentifierType.Ric,
					    }
			    };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EndOfDayPricingReportTemplate", "Create");
            //Define the report template specific to the desired content set
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "example-eod" + Suffix.New(),
            };

            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "52 Week High"},
                new ContentField { FieldName = "7 Day Compound Yield"},
                new ContentField { FieldName = "Ask Price"},
                new ContentField { FieldName = "Close Price"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create");
            //Define the schedule to be created
            var scheduleName = "example-eod" + Suffix.New();
            var schedule = new Schedule
            {
                Name = scheduleName,
                TimeZone = "Central Standard Time",
                Recurrence = ScheduleRecurrence.CreateWeeklyRecurrence(
                    new[] { DayOfWeek.Monday, DayOfWeek.Friday }),
                Trigger = ScheduleTrigger.CreateTimeTrigger(
                    false, //Limit to todays data
                    new[] { new HourMinute { Hour = 16, Minute = 0 } }),
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "GetByName");
            //Fetch the schedule by name
            var scheduleToUpdate = ExtractionsContext.ScheduleOperations.GetByName(scheduleName);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Update", MethodType.Operation, Publish.Primary);
            //Change the schedule to trigger as soon as data is available on the scheduled date.
            scheduleToUpdate.Trigger = new DataAvailabilityTrigger
            {
                ExtendedInformation = new EndOfDayExtendedTriggerInformation
                {
                    IsErrorCorrected = true,
                    IsDelta = true,
                },
                LimitReportToTodaysData = false
            };

            //Change the schedule to run on the 15th of every month
            schedule.Recurrence = ScheduleRecurrence.CreateMonthlyRecurrence(15);

            //Update the schedule
            ExtractionsContext.ScheduleOperations.Update(schedule);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Delete

        /// <summary>
        /// This example shows how to delete a Schedule.
        /// </summary>
        [Example("Delete")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(Schedule))]
        public void DeleteSchedule()
        {
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "example-eod" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
			    {
				    new InstrumentIdentifier
					    {
						    Identifier = "IBM.N",
						    IdentifierType = IdentifierType.Ric,
						    UserDefinedIdentifier = "UserIdent1A", //User identifiers are optional
						    UserDefinedIdentifier2 = "UserIdent1B",
						    UserDefinedIdentifier3 = "UserIdent1C"
					    },
				    new InstrumentIdentifier
					    {
						    Identifier = "KO.N",
						    IdentifierType = IdentifierType.Ric,
					    }
			    };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EndOfDayPricingReportTemplate", "Create");
            //Define the report template specific to the desired content set
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "example-eod" + Suffix.New(),
            };

            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "52 Week High"},
                new ContentField { FieldName = "7 Day Compound Yield"},
                new ContentField { FieldName = "Ask Price"},
                new ContentField { FieldName = "Close Price"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create");
            //Define the schedule to be created
            var scheduleName = "example-eod" + Suffix.New();
            var schedule = new Schedule
            {
                Name = scheduleName,
                TimeZone = "Central Standard Time",
                Recurrence = ScheduleRecurrence.CreateWeeklyRecurrence(
                    new[] { DayOfWeek.Monday, DayOfWeek.Friday }),
                Trigger = ScheduleTrigger.CreateTimeTrigger(
                    false, //Limit to todays data
                    new[] { new HourMinute { Hour = 16, Minute = 0 } }),
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "GetByName");
            //Fetch the schedule by name
            var scheduleToDelete = ExtractionsContext.ScheduleOperations.GetByName(scheduleName);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Delete", MethodType.Operation, Publish.Primary);
            //Delete the schedule.
            ExtractionsContext.ScheduleOperations.Delete(scheduleToDelete);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Cancel

        [Example("Cancel By Schedule (Tick History Only)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(Schedule))]
        public void CancelScheduleExtractions()
        {
            /* Scheduled Tick History extractions are cancellable if they are currently processing and they are not in the 
             * delivery phase.
             * 
             * Invoking the cancellation request will attempt to cancel all extractions that are currently in process.  If 
             * an extraction has not yet started it will not be cancelled.  If an extraction has completed or errored out, 
             * it will not be cancelled.  Standard billing rates apply for extractions that could not be cancelled.
             * 
             * This example demonstrates how to create a tick history time and sales report template, schedule it for
             * immediate extraction and then attempt to cancel the extraction.  If the report template is processing and 
             * it supports cancellation, the cancellation request will be accepted, but it is still not certain that the 
             * request will be cancelled since it may be in the final stages of delivery.  It is the caller's 
             * responsibility to check the status of the extraction to determine if the extraction was cancelled or 
             * completed.
             */
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "example-eod" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
			    {
				    new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
				    new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
			    };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "TickHistoryTimeAndSalesReportTemplate", "Create");
            //Define the report template specific to the desired content set
            var reportTemplate = new TickHistoryTimeAndSalesReportTemplate
            {
                Name = "example-cancel" + Suffix.New(),
                Condition = new TickHistoryTimeAndSalesCondition
                {
                    QueryStartDate = new DateTimeOffset(new DateTime(2015, 12, 01, 00, 00, 00)),
                    QueryEndDate = new DateTimeOffset(new DateTime(2015, 12, 04, 23, 59, 59)),
                    MessageTimeStampIn = TickHistoryTimeOptions.GmtUtc,
                    ReportDateRangeType = ReportDateRangeType.Range,
                    ExtractBy = TickHistoryExtractByMode.Ric,
                    SortBy = TickHistorySort.SingleByRic,
                }
            };
            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "Quote - Ask Price"},
                new ContentField { FieldName = "Quote - Ask Size"},
                new ContentField { FieldName = "Quote - Ask Yield"},
                new ContentField { FieldName = "Quote - Bid Price"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create");
            //Define the schedule to be created.  The schedule is set to trigger immediately.
            var scheduleName = "example-eod" + Suffix.New();
            var schedule = new Schedule
            {
                Name = scheduleName,
                TimeZone = "Central Standard Time",
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow, true),
                Trigger = ScheduleTrigger.CreateImmediateTrigger(false),
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "RequestCancellationOfInProcessExtractions");
            //An attempt is made to cancel all currently running extractions for this schedule.
            //If no extractions are in progress a 400 / ValidationException will be returned.  If extractions
            //are in progress an attempt will be made to stop the extractions, but it is possible the extraction
            //will complete and usage/billing will be recorded as a successful transaction.  Cancellation is 
            //only supported for Tick History report templates.
            try
            {
                ExtractionsContext.ScheduleOperations.RequestCancellationOfInProcessExtractions(schedule);
            }
            catch (ValidationException ex)
            {
                Status.WriteLine("Cancellation rejected: {0}", ex.Message);
            }
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "ReportExtractionOperations", "Get", MethodType.Operation, Publish.Secondary);
            //Optional: You can poll to determine the status of the extractions for the schedule to determine if the 
            //cancellation was accepted
            var reportExtractions = ExtractionsContext.ReportExtractionOperations.GetByScheduleId(schedule.ScheduleId);
            foreach (var reportExtraction in reportExtractions)
                Status.WriteLine("Report Extraction date: {0} status: {1}, detailed status: {2}",
                    reportExtraction.ExtractionDateUtc, reportExtraction.Status, reportExtraction.DetailedStatus);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Extractions

        /// <summary>
        /// This example starts an immediate extraction for the schedule.  It does not wait for the extraction to complete.
        /// Note, if the cleanup is allowed to run, it will delete the schedule and prevent the extraction from completing.
        /// </summary>
        [Example("Request Adhoc Extraction")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(Schedule))]
        public void StartAnExtractionForASchedule()
        {
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "example-eod" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
			    {
				    new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
				    new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
			    };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EndOfDayPricingReportTemplate", "Create");
            //Define the report template specific to the desired content set
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "example-eod" + Suffix.New(),
            };

            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "52 Week High"},
                new ContentField { FieldName = "7 Day Compound Yield"},
                new ContentField { FieldName = "Ask Price"},
                new ContentField { FieldName = "Close Price"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create");
            //Define the schedule to be created
            var scheduleName = "example-eod" + Suffix.New();
            var schedule = new Schedule
            {
                Name = scheduleName,
                TimeZone = "Central Standard Time",
                Recurrence = ScheduleRecurrence.CreateWeeklyRecurrence(
                    new[] { DayOfWeek.Monday, DayOfWeek.Friday }),
                Trigger = ScheduleTrigger.CreateTimeTrigger(
                    false, //Limit to todays data
                    new[] { new HourMinute { Hour = 16, Minute = 0 } }),
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "ExtractNow", MethodType.Operation, Publish.Primary);
            //Force an extraction of this schedule.  The results are returned when the extraction 
            //has been completed.  ExtractNow is asynchronous which means that if the extraction
            //takes more than 30 seconds, the client will poll the server until the results are
            //available.  See ReportExtractions examples for details on retrieving the contents
            //of an extraction.
            ExtractionsContext.ScheduleOperations.ExtractNow(schedule);
            var reportExtraction = ExtractionsContext.ScheduleOperations.WaitForNextExtraction(schedule);

            Status.WriteLine("Schedule: {0}", reportExtraction.ScheduleName);
            Status.WriteLine("Status: {0}", reportExtraction.Status); //Completed, Pending, Processing
            Status.WriteLine("Detailed Status: {0}", reportExtraction.DetailedStatus);
            Status.EndNotify(ExtractionsContext);
        }

        /// <summary>
        /// This example shows how to get the last extraction that was completed for the schedule.
        /// </summary>
        [Example("Get Last Extraction")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(Schedule))]
        public void GetLastExtraction()
        {
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "example-eod" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
			    {
				    new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
				    new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
			    };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EndOfDayPricingReportTemplate", "Create");
            //Define the report template specific to the desired content set
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "example-eod" + Suffix.New(),
            };

            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "52 Week High"},
                new ContentField { FieldName = "7 Day Compound Yield"},
                new ContentField { FieldName = "Ask Price"},
                new ContentField { FieldName = "Close Price"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create");
            //Define the schedule to be created
            var scheduleName = "example-eod" + Suffix.New();
            var schedule = new Schedule
            {
                Name = scheduleName,
                TimeZone = "Central Standard Time",
                Recurrence = ScheduleRecurrence.CreateWeeklyRecurrence(
                    new[] { DayOfWeek.Monday, DayOfWeek.Friday }),
                Trigger = ScheduleTrigger.CreateTimeTrigger(
                    false, //Limit to todays data
                    new[] { new HourMinute { Hour = 16, Minute = 0 } }),
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "ExtractNow");
            //Force an extraction of this schedule.  The results are returned when the extraction 
            //has been completed.  ExtractNow is asynchronous which means that if the extraction
            //takes more than 30 seconds, the client will poll the server until the results are
            //available.  See ReportExtractions examples for details on retrieving the contents
            //of an extraction.
            ExtractionsContext.ScheduleOperations.ExtractNow(schedule);
            var reportExtraction = ExtractionsContext.ScheduleOperations.WaitForNextExtraction(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "LastExtraction", MethodType.Association, Publish.Primary);
            //Returns details regarding the last extraction (this may be null)
            ExtractionsContext.LoadProperty(schedule, "LastExtraction");  //Loads Schedule.LastExtraction property

            //If there is a last extraction, output the status
            if (schedule.LastExtraction != null)
            {
                Status.WriteLine("Schedule: {0}", schedule.LastExtraction.ScheduleName);
                Status.WriteLine("Status: {0}", schedule.LastExtraction.Status); //Completed, Pending, Processing
                Status.WriteLine("Detailed Status: {0}", schedule.LastExtraction.DetailedStatus);
            }
            Status.EndNotify(ExtractionsContext);
        }

        /// <summary>
        /// This example shows how to get the next pending extraction for the schedule.
        /// </summary>
        [Example("Get the Next (Pending) Extraction for this Schedule")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(Schedule))]
        public void GetNextExtraction()
        {
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "example-eod" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
			    {
				    new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
				    new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
			    };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EndOfDayPricingReportTemplate", "Create");
            //Define the report template specific to the desired content set
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "example-eod" + Suffix.New(),
            };

            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "52 Week High"},
                new ContentField { FieldName = "7 Day Compound Yield"},
                new ContentField { FieldName = "Ask Price"},
                new ContentField { FieldName = "Close Price"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create");
            //Define the schedule to be created
            var scheduleName = "example-eod" + Suffix.New();
            var schedule = new Schedule
            {
                Name = scheduleName,
                TimeZone = "Central Standard Time",
                Recurrence = ScheduleRecurrence.CreateWeeklyRecurrence(
                    new[] { DayOfWeek.Monday, DayOfWeek.Friday }),
                Trigger = ScheduleTrigger.CreateTimeTrigger(
                    false, //Limit to todays data
                    new[] { new HourMinute { Hour = 16, Minute = 0 } }),
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "NextExtraction", MethodType.Operation, Publish.Primary);
            //Returns details regarding the last extraction (this may be null)
            ExtractionsContext.LoadProperty(schedule, "NextExtraction");  //Loads Schedule.NextExtraction property

            //If there is a last extraction, output the status
            if (schedule.NextExtraction != null)
            {
                Status.WriteLine("Schedule: {0}", schedule.NextExtraction.ScheduleName);
                Status.WriteLine("Status: {0}", schedule.NextExtraction.Status); //Completed, Pending, Processing
                Status.WriteLine("Detailed Status: {0}", schedule.NextExtraction.DetailedStatus);
            }
            Status.EndNotify(ExtractionsContext);
        }

        /// <summary>
        /// This example shows how to get the last extraction that was completed for the schedule.
        /// </summary>
        [Example("Get Completed Extractions for a Schedule")]
        public void GetCompletedExtractions()
        {
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "example-eod" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
			    {
				    new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
				    new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
			    };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EndOfDayPricingReportTemplate", "Create");
            //Define the report template specific to the desired content set
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "example-eod" + Suffix.New(),
            };

            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "52 Week High"},
                new ContentField { FieldName = "7 Day Compound Yield"},
                new ContentField { FieldName = "Ask Price"},
                new ContentField { FieldName = "Close Price"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create");
            //Define the schedule to be created
            var scheduleName = "example-eod" + Suffix.New();
            var schedule = new Schedule
            {
                Name = scheduleName,
                TimeZone = "Central Standard Time",
                Recurrence = ScheduleRecurrence.CreateWeeklyRecurrence(
                    new[] { DayOfWeek.Monday, DayOfWeek.Friday }),
                Trigger = ScheduleTrigger.CreateTimeTrigger(
                    false, //Limit to todays data
                    new[] { new HourMinute { Hour = 16, Minute = 0 } }),
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "ExtractNow");
            //Force an extraction of this schedule
            ExtractionsContext.ScheduleOperations.ExtractNow(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "CompletedExtractions", MethodType.Association, Publish.Primary);
            //Load the Schedule.CompletedExtractions property by fetching contents
            //from the server.
            ExtractionsContext.LoadProperty(schedule, "CompletedExtractions");

            //Output the status for each completed extractions for this schedule
            foreach (var extraction in schedule.CompletedExtractions)
                Status.WriteLine("Extraction {0} status = {1}", extraction.ReportExtractionId, extraction.Status);
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

        #region Export / Import

        [Example("Export Xml")]
        public void ExportXml()
        {
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "export-xml" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
			    {
				    new InstrumentIdentifier
					    {
						    Identifier = "IBM.N",
						    IdentifierType = IdentifierType.Ric
                        }
			    };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EndOfDayPricingReportTemplate", "Create");
            //Define the report template specific to the desired content set
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "export-xml" + Suffix.New(),
            };

            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "52 Week High"},
                new ContentField { FieldName = "7 Day Compound Yield"},
                new ContentField { FieldName = "Ask Price"},
                new ContentField { FieldName = "Close Price"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create");
            //Define the schedule to be created
            var schedule = new Schedule
            {
                Name = "export-xml" + Suffix.New(),
                TimeZone = "Central Standard Time",
                Recurrence = ScheduleRecurrence.CreateWeeklyRecurrence(
                    new[] { DayOfWeek.Monday, DayOfWeek.Friday }),
                Trigger = ScheduleTrigger.CreateTimeTrigger(
                    false, //Limit to todays data
                    new[] { new HourMinute { Hour = 16, Minute = 0 } }),
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EntityList", "Export", MethodType.Operation, Publish.Primary);
            var exportBytes = ExtractionsContext.ScheduleOperations.Export(
                schedule.ScheduleId, true /* include header */);
            var exportString = Encoding.UTF8.GetString(exportBytes);

            //Output
            Status.WriteLine(exportString);
            Status.EndNotify(ExtractionsContext);

        }

        [Example("Import Xml")]
        public void ImportXml()
        {
            CleanUp();  // cleanup in case previously called

            Status.Notify(ExtractionsContext, "InstrumentList", "Create");
            //Create a new instrument list
            var instrumentList = new InstrumentList { Name = "export-xml" + Suffix.New() };
            ExtractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new List<InstrumentIdentifier>
			    {
				    new InstrumentIdentifier
					    {
						    Identifier = "IBM.N",
						    IdentifierType = IdentifierType.Ric
                        }
			    };
            ExtractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EndOfDayPricingReportTemplate", "Create");
            //Define the report template specific to the desired content set
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "export-xml" + Suffix.New(),
            };

            //Define the fields to be queried.  See GetAvailableFields example.
            reportTemplate.ContentFields.AddRange(new[] {
                new ContentField { FieldName = "52 Week High"},
                new ContentField { FieldName = "7 Day Compound Yield"},
                new ContentField { FieldName = "Ask Price"},
                new ContentField { FieldName = "Close Price"}});

            //Post the request to create the new template
            ExtractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "Schedule", "Create");
            //Define the schedule to be created
            var schedule = new Schedule
            {
                Name = "export-xml" + Suffix.New(),
                TimeZone = "Central Standard Time",
                Recurrence = ScheduleRecurrence.CreateWeeklyRecurrence(
                    new[] { DayOfWeek.Monday, DayOfWeek.Friday }),
                Trigger = ScheduleTrigger.CreateTimeTrigger(
                    false, //Limit to todays data
                    new[] { new HourMinute { Hour = 16, Minute = 0 } }),
                ListId = instrumentList.ListId, //Reference the list of instruments
                ReportTemplateId = reportTemplate.ReportTemplateId, //Reference the report template
            };

            //Post the request to create the new schedule.
            ExtractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EntityList", "Export", MethodType.Operation, Publish.Secondary);
            var exportBytes = ExtractionsContext.ScheduleOperations.Export(
                schedule.ScheduleId, true /* include header */);
            var exportString = Encoding.UTF8.GetString(exportBytes);

            //Output
            Status.WriteLine(exportString);
            Status.EndNotify(ExtractionsContext);

            Status.Notify(ExtractionsContext, "EntityList", "Import", MethodType.Operation, Publish.Primary);
            var importBytes = Encoding.UTF8.GetBytes(exportString);
            ExtractionsContext.ScheduleOperations.Import(
                importBytes, new ScheduleImportSettings { Replace = true });
            Status.EndNotify(ExtractionsContext);
        }

        #endregion

    }

}
