﻿
using System;
using System.Linq;
using Dss.Api.Examples.Internal;
using DataScope.Select.Api.Extractions.SubjectLists;
using System.Collections.Generic;
using DataScope.Select.Api.Extractions;
using DataScope.Select.Core.RestApi;

namespace Dss.Api.Examples.Extractions.SubjectLists.CriteriaLists
{
    [Example("Criteria List Examples")]
    class CriteriaListExamples
    {
        [Example("Create Without Preferred Identifier Type")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(CriteriaList))]
        public void Create()
        {
            // Create context
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "CriteriaList", "Create", MethodType.Operation, Publish.Primary);
            // Create a loan criteria list applying filters Active Status, Currency Code, Bid Price, and Maturity
            var filters = new List<CriteriaListFilter>
            {
                new BooleanFilter
                {
                    Name = "Active Status",
                    Value = BooleanValue.Yes
                },
                new MultiValueFilter
                {
                    Name = "Currency Code",
                    Op = MultiValueOperator.In,
                    Values = new DssCollection<string>
                    {
                        "GBP"
                    }
                },
                new RangeFilter
                {
                    Name = "Bid Price",
                    Op = RangeOperator.Between,
                    Min = (decimal?)98.0,
                    Max = (decimal?)100.0
                },
                new DateRangeFilter
                {
                    Name = "Maturity",
                    Op = RangeOperator.GreaterThan,
                    Value = new DateTimeOffset(new DateTime(2017, 6, 1))
                }
            };
            var criteriaList = extractionsContext.CriteriaListOperations.Create("LoansCriteriaList" + Suffix.New(), CriteriaListType.Loans, filters);

            Status.WriteLine("Total filters applied: " + criteriaList.Filters.Count);
            foreach (var filter in criteriaList.Filters)
            {
                Status.WriteLine(filter.Name + " is applied to the criteria list.");
            }
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "CriteriaList", "Exists", MethodType.Operation, Publish.Secondary);
            // Check if created criteria list exists in DSS
            var listExists = extractionsContext.CriteriaListOperations.Exists(criteriaList.ListId);
            Status.WriteLine("Does the criteria list exist? " + (listExists ? "Yes." : "No."));
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "CriteriaList", "Delete", MethodType.Operation, Publish.Secondary);
            // Delete the criteria list
            extractionsContext.CriteriaListOperations.Delete(criteriaList);
            Status.EndNotify(extractionsContext);
        }


        [Example("Create With Preferred Identifier Type")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(CriteriaList))]
        public void CreateWithPreferredId()
        {
            // Create context
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "CriteriaList", "CreateWithPreferredIdentifierType", MethodType.Operation, Publish.Primary);
            // Create a MiFID Sub-Class criteria list applying filters Asset Class, Sub-Asset Class, Time to Maturity Bucket, and Post Trade SSTI Threshold Value
            var filters = new List<CriteriaListFilter>
            {
                new SingleValueFilter
                {
                    Name = "Asset Class",
                    Value = "CMDV" // Commodity Derivatives
                },
                new SingleValueFilter
                {
                    Name = "Sub-Asset Class",
                    Value = "ACFF" // Agriculture Commodity Futures/Forwards
                },
                new RangeFilter
                {
                    Name = "Post Trade SSTI Threshold Value",
                    Op = RangeOperator.LessThanOrEqual,
                    Value = 750000
                },
                new MultiValueFilter
                {
                    Name = "Time to Maturity Bucket",
                    Op = MultiValueOperator.In,
                    Values = new DssCollection<string>
                    {
                        "TTM_3M_6M"
                    }
                }
            };
            var criteriaList = extractionsContext.CriteriaListOperations.Create("MifidSubclassCriteriaList" + Suffix.New(), CriteriaListType.MifidSubclass, filters, DataScope.Select.Api.Content.IdentifierType.Pix);

            Status.WriteLine("Total filters applied: " + criteriaList.Filters.Count);
            foreach (var filter in criteriaList.Filters)
            {
                Status.WriteLine(filter.Name + " is applied to the criteria list.");
            }
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "CriteriaList", "Exists", MethodType.Operation, Publish.Secondary);
            // Check if created criteria list exists in DSS
            var listExists = extractionsContext.CriteriaListOperations.Exists(criteriaList.ListId);
            Status.WriteLine("Does the criteria list exist? " + (listExists ? "Yes." : "No."));
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "CriteriaList", "Delete", MethodType.Operation, Publish.Secondary);
            // Delete the criteria list
            extractionsContext.CriteriaListOperations.Delete(criteriaList);
            Status.EndNotify(extractionsContext);
        }


        [Example("Create Fixed Income New Issuance Criteria List")]
        public void CreateFixedIncomeNewIssuanceCriteriaList()
        {
            // Create context
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "CriteriaList", "Create", MethodType.Operation, Publish.Secondary);

            // Create a Fixed Income New Issuance criteria list applying multiple filters
            var filters = new List<CriteriaListFilter>
            {
                new RangeFilter
                {
                    Name = "Coupon Rate",
                    Op = RangeOperator.GreaterThan,
                    Value = (decimal?)6.95
                },
                new DateRangeFilter
                {
                    Name = "Issue Date",
                    Op = RangeOperator.GreaterThan,
                    Value = new DateTimeOffset(new DateTime(2022, 01, 01))
                },
                new DateRangeFilter
                {
                    Name = "Maturity Date",
                    Op = RangeOperator.Between,
                    Min = new DateTimeOffset(new DateTime(2022, 08, 01)),
                    Max = new DateTimeOffset(new DateTime(2022, 08, 31))
                },
                new RangeFilter
                {
                    Name = "Term To Maturity In Days",
                    Value = 1050
                }
            };
            var criteriaList = extractionsContext.CriteriaListOperations.Create("FixedIncomeNewIssuanceCriteriaList" + Suffix.New(), CriteriaListType.FixedIncomeNewIssuance, filters);

            Status.WriteLine("Total filters applied: " + criteriaList.Filters.Count);
            foreach (var filter in criteriaList.Filters)
            {
                Status.WriteLine(filter.Name + " is applied to the criteria list.");
            }
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "CriteriaList", "Exists", MethodType.Operation, Publish.Secondary);
            // Check if created criteria list exists in DSS
            var listExists = extractionsContext.CriteriaListOperations.Exists(criteriaList.ListId);
            Status.WriteLine("Does the criteria list exist? " + (listExists ? "Yes." : "No."));
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "CriteriaList", "Delete", MethodType.Operation, Publish.Secondary);
            // Delete the criteria list
            extractionsContext.CriteriaListOperations.Delete(criteriaList);
            Status.EndNotify(extractionsContext);
        }

        [Example("Create MiFID Sub-Class Criteria List")]
        public void CreateMifidSubclassCriteriaList()
        {
            // Create context
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "CriteriaList", "Create", MethodType.Operation, Publish.Secondary);
            // Create a MiFID Sub-Class criteria list applying filters Asset Class, Sub-Asset Class, Time to Maturity Bucket, and Post Trade SSTI Threshold Value
            var filters = new List<CriteriaListFilter>
            {
                new SingleValueFilter
                {
                    Name = "Asset Class",
                    Value = "CMDV" // Commodity Derivatives
                },
                new SingleValueFilter
                {
                    Name = "Sub-Asset Class",
                    Value = "ACFF" // Agriculture Commodity Futures/Forwards
                },
                new RangeFilter
                {
                    Name = "Post Trade SSTI Threshold Value",
                    Op = RangeOperator.LessThanOrEqual,
                    Value = 750000
                },
                new MultiValueFilter
                {
                    Name = "Time to Maturity Bucket",
                    Op = MultiValueOperator.In,
                    Values = new DssCollection<string>
                    {
                        "TTM_3M_6M"
                    }
                }
            };
            var criteriaList = extractionsContext.CriteriaListOperations.Create("MifidSubclassCriteriaList" + Suffix.New(), CriteriaListType.MifidSubclass, filters);

            Status.WriteLine("Total filters applied: " + criteriaList.Filters.Count);
            foreach (var filter in criteriaList.Filters)
            {
                Status.WriteLine(filter.Name + " is applied to the criteria list.");
            }
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "CriteriaList", "Exists", MethodType.Operation, Publish.Secondary);
            // Check if created criteria list exists in DSS
            var listExists = extractionsContext.CriteriaListOperations.Exists(criteriaList.ListId);
            Status.WriteLine("Does the criteria list exist? " + (listExists ? "Yes." : "No."));
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "CriteriaList", "Delete", MethodType.Operation, Publish.Secondary);
            // Delete the criteria list
            extractionsContext.CriteriaListOperations.Delete(criteriaList);
            Status.EndNotify(extractionsContext);
        }

        [Example("Create Publicly-Traded Partnership (PTP) Criteria List")]
        public void CreatePubliclyTradedPartnershipCriteriaList()
        {
            // Create context
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "CriteriaList", "Create", MethodType.Operation, Publish.Secondary);

            // Create a Publicly-Traded Partnership (PTP) criteria list applying multiple filters
            var filters = new List<CriteriaListFilter>
            {
                new BooleanFilter
                {
                    Name = "Active Status",
                    Value = BooleanValue.All
                },
                new SingleValueFilter
                {
                    Name = "Asset Type",
                    Value = "EQTY"
                },
                new MultiValueFilter
                {
                    Name = "Publicly-Traded Partnership (PTP)",
                    Values = new [] { "1" }
                }
            };
            var criteriaList = extractionsContext.CriteriaListOperations.Create("PubliclyTradedPartnershipCriteriaList" + Suffix.New(), CriteriaListType.PubliclyTradedPartnership, filters);

            Status.WriteLine("Total filters applied: " + criteriaList.Filters.Count);
            foreach (var filter in criteriaList.Filters)
            {
                Status.WriteLine(filter.Name + " is applied to the criteria list.");
            }
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "CriteriaList", "Exists", MethodType.Operation, Publish.Secondary);
            // Check if created criteria list exists in DSS
            var listExists = extractionsContext.CriteriaListOperations.Exists(criteriaList.ListId);
            Status.WriteLine("Does the criteria list exist? " + (listExists ? "Yes." : "No."));
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "CriteriaList", "Delete", MethodType.Operation, Publish.Secondary);
            // Delete the criteria list
            extractionsContext.CriteriaListOperations.Delete(criteriaList);
            Status.EndNotify(extractionsContext);
        }

        [Example("Update Filters Without Preferred Identifier Type")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(CriteriaList))]
        public void UpdateFilters()
        {
            // Create context
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "CriteriaList", "Create");
            // Create a commodities criteria list applying universe filters (Active Status & Basis Gap Adjusted) 
            // and filters Currencies, Barrels Per Ton, and Expiration Date
            var filters = new List<CriteriaListFilter>
            {
                new BooleanFilter
                {
                    Name = "Active Status",
                    Value = BooleanValue.Yes
                },
                new BooleanFilter
                {
                    Name = "Basis Gap Adjusted",
                    Value = BooleanValue.All
                },
                new MultiValueFilter
                {
                    Name = "Currencies",
                    Op = MultiValueOperator.In,
                    Values = new DssCollection<string>
                    {
                        "USD"
                    }
                },
                new RangeFilter
                {
                    Name = "Barrels Per Ton",
                    Op = RangeOperator.Between,
                    Min = (decimal?)7.28,
                    Max = (decimal?)8.46
                },
                new DateRangeFilter
                {
                    Name = "Expiration Date",
                    Op = RangeOperator.GreaterThan,
                    Value = new DateTimeOffset(new DateTime(2017, 6, 1))
                }
            };
            var criteriaList = new CriteriaList
            {
                Name = "CriteriaListUpdate" + Suffix.New(),
                Type = CriteriaListType.Commodities,
                Filters = new DssCollection<CriteriaListFilter>(filters)
            };
            extractionsContext.CriteriaListOperations.Create(criteriaList);

            Status.WriteLine("CriteriaList ListId: {0}", criteriaList.ListId);
            foreach (var filter in criteriaList.Filters)
            {
                Status.WriteLine("Filter applied: {0}", filter.Name);
            }
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "CriteriaList", "UpdateFilters", MethodType.Operation, Publish.Secondary);
            var updatedFilters = criteriaList.Filters.ToList();
            // Add "Underlying Asset"
            updatedFilters.Add(new MultiValueFilter
            {
                Name = "Underlying Asset",
                Op = MultiValueOperator.In,
                Values = new DssCollection<string> { "Liquid Propane Gas" }
            });

            // Modify "Barrels Per Ton"
            var bptFilter = (RangeFilter)updatedFilters.First(f => f.Name == "Barrels Per Ton");
            bptFilter.Op = RangeOperator.GreaterThan;
            bptFilter.Value = (decimal) 8.0;

            // Remove "Expiration Date"
            updatedFilters.RemoveAll(f => f.Name == "Expiration Date");

            // Apply new filters to list and update
            criteriaList.Filters = new DssCollection<CriteriaListFilter>(updatedFilters);
            extractionsContext.CriteriaListOperations.UpdateFilters(criteriaList);
            Status.EndNotify(extractionsContext);
            
            // Get list from server to verify filters were updated
            Status.Notify(extractionsContext, "CriteriaList", "Get", MethodType.Operation, Publish.Secondary);
            var fromServer = extractionsContext.CriteriaListOperations.Get(criteriaList.ListId);
            foreach (var filter in fromServer.Filters)
            {
                Status.WriteLine("Filter applied: {0}", filter.Name);
            }
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "CriteriaList", "Delete");
            // Delete the criteria list
            extractionsContext.CriteriaListOperations.Delete(criteriaList);
            Status.EndNotify(extractionsContext);
        }

        [Example("Update Filters With Preferred Identifier Type")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(CriteriaList))]
        public void UpdateWithPreferredId()
        {
            // Create context
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "CriteriaList", "CreateWithPreferredIdentifierType", MethodType.Operation, Publish.Primary);

            // Create a MiFID Sub-Class criteria list applying filters Asset Class, Sub-Asset Class, Time to Maturity Bucket, and Post Trade SSTI Threshold Value
            var filters = new List<CriteriaListFilter>
            {
                new SingleValueFilter
                {
                    Name = "Asset Class",
                    Value = "CMDV" // Commodity Derivatives
                },
                new SingleValueFilter
                {
                    Name = "Sub-Asset Class",
                    Value = "ACFF" // Agriculture Commodity Futures/Forwards
                },
                new RangeFilter
                {
                    Name = "Post Trade SSTI Threshold Value",
                    Op = RangeOperator.LessThanOrEqual,
                    Value = 750000
                },
                new MultiValueFilter
                {
                    Name = "Time to Maturity Bucket",
                    Op = MultiValueOperator.In,
                    Values = new DssCollection<string>
                    {
                        "TTM_3M_6M"
                    }
                }
            };
            var criteriaList = extractionsContext.CriteriaListOperations.Create("MifidSubclassCriteriaList" + Suffix.New(), CriteriaListType.MifidSubclass, filters, DataScope.Select.Api.Content.IdentifierType.Pix);


            Status.WriteLine("CriteriaList ListId: {0}", criteriaList.ListId);
            foreach (var filter in criteriaList.Filters)
            {
                Status.WriteLine("Filter applied: {0}", filter.Name);
            }
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "CriteriaList", "UpdateFiltersWithPreferredIdentifierType", MethodType.Operation, Publish.Primary);
            var updatedFilters = criteriaList.Filters.ToList();

            // Modify "Post Trade SSTI Threshold Value"
            var bptFilter = (RangeFilter)updatedFilters.First(f => f.Name == "Post Trade SSTI Threshold Value");
            bptFilter.Op = RangeOperator.GreaterThan;
            bptFilter.Value = 760000;

            // Apply new filters to list and update
            criteriaList.Filters = new DssCollection<CriteriaListFilter>(updatedFilters);

            extractionsContext.CriteriaListOperations.UpdateFilters(criteriaList, DataScope.Select.Api.Content.IdentifierType.Pid);

            Status.EndNotify(extractionsContext);

            // Get list from server to verify filters were updated
            Status.Notify(extractionsContext, "CriteriaList", "Get", MethodType.Operation, Publish.Primary);
            var fromServer = extractionsContext.CriteriaListOperations.Get(criteriaList.ListId);
            foreach (var filter in fromServer.Filters)
            {
                if(filter.Name == "Post Trade SSTI Threshold Value")
                {
                    RangeFilter rf = (RangeFilter)filter;
                    Status.WriteLine("Filter applied:  {0} (value = '{1}{2}')", filter.Name, rf.Op, rf.Value);
                }
                else
                    Status.WriteLine("Filter applied: {0}", filter.Name);
            }

            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "CriteriaList", "Delete");
            // Delete the criteria list
            extractionsContext.CriteriaListOperations.Delete(criteriaList);
            Status.EndNotify(extractionsContext);
        }

        [Example("GetByName")]
        public void GetByName()
        {
            // Create context
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "CriteriaList", "Create");

            // Create a MiFID Sub-Class criteria list applying filters Asset Class, Sub-Asset Class, Time to Maturity Bucket, and Post Trade SSTI Threshold Value
            var filters = new List<CriteriaListFilter>
            {
                new SingleValueFilter
                {
                    Name = "Asset Class",
                    Value = "CMDV" // Commodity Derivatives
                },
                new SingleValueFilter
                {
                    Name = "Sub-Asset Class",
                    Value = "ACFF" // Agriculture Commodity Futures/Forwards
                },
                new RangeFilter
                {
                    Name = "Post Trade SSTI Threshold Value",
                    Op = RangeOperator.LessThanOrEqual,
                    Value = 750000
                },
                new MultiValueFilter
                {
                    Name = "Time to Maturity Bucket",
                    Op = MultiValueOperator.In,
                    Values = new DssCollection<string>
                    {
                        "TTM_3M_6M"
                    }
                }
            };

            string listName = "MifidSubclassCriteriaList" + Suffix.New();
            var criteriaList = extractionsContext.CriteriaListOperations.Create(listName, CriteriaListType.MifidSubclass, filters, DataScope.Select.Api.Content.IdentifierType.Pix);
            Status.WriteLine(string.Format("Created CriterialList '{0}' with Preferred Identifier Type '{1}]", listName, criteriaList.PreferredIdentifierType));

            var gotten = extractionsContext.CriteriaListOperations.GetByName(listName);
            Status.WriteLine(string.Format("Retrieved CriterialList '{0}' with Preferred Identifier Type '{1}]", listName, gotten.PreferredIdentifierType));

            var g = extractionsContext.CriteriaListOperations.Get(gotten.ListId);

            var glist = extractionsContext.CriteriaListOperations.GetAll();

            extractionsContext.CriteriaListOperations.Delete(gotten);
            Status.WriteLine(string.Format("Deleted CriterialList '{0}'.", listName));

            

            Status.EndNotify(extractionsContext);

        }

        [Example("Copy")]
        public void Copy()
        {
            // Create context
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            // Create a criteria list
            var filters = new List<CriteriaListFilter>
            {
                new BooleanFilter
                {
                    Name = "Active Status",
                    Value = BooleanValue.Yes
                },
                new MultiValueFilter
                {
                    Name = "Currency Code",
                    Op = MultiValueOperator.In,
                    Values = new DssCollection<string>
                    {
                        "GBP"
                    }
                }
            };
            var criteriaList = extractionsContext.CriteriaListOperations.Create("LoansCriteriaList" + Suffix.New(), CriteriaListType.Loans, filters);

            Status.Notify(extractionsContext, "CriteriaList", "Copy", MethodType.Operation, Publish.Secondary);
            // Copy criteria list
            const string copyListName = "CopyOfCriteriaList";
            var listCopy = extractionsContext.CriteriaListOperations.Copy(criteriaList, copyListName);

            // Copy verification
            Status.WriteLine("Copied list name: " + listCopy.Name);
            Status.WriteLine("Original List Filter Count: " + criteriaList.Filters.Count);
            Status.WriteLine("Copied list filter count: " + listCopy.Filters.Count);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "CriteriaList", "DeleteMultiple", MethodType.Operation, Publish.Secondary);
            // Delete multiple criteria lists
            extractionsContext.CriteriaListOperations.DeleteMultiple(new[] { criteriaList.ListId, listCopy.ListId });
            Status.EndNotify(extractionsContext);
        }

        [Example("GetAll")]
        public void GetAll()
        {
            // Create context
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            //Set the page size to 1 record per page for the purposes of this example
            extractionsContext.Preferences.MaxPageSize = 1;

            // Create a couple criteria lists for the example
            var filters = new List<CriteriaListFilter>
            {
                new BooleanFilter { Name = "Active Status", Value = BooleanValue.Yes },
                new BooleanFilter { Name = "Basis Gap Adjusted", Value = BooleanValue.All }
            };
            var criteriaList1 = extractionsContext.CriteriaListOperations.Create("CommoditiesCriteriaList" + Suffix.New(), CriteriaListType.Commodities, filters);
            var criteriaList2 = extractionsContext.CriteriaListOperations.Create("CommoditiesCriteriaList" + Suffix.New(), CriteriaListType.Commodities, filters);

            Status.Notify(extractionsContext, "CriteriaList", "GetAll", MethodType.Operation, Publish.Secondary);
            //Get the first page of instrument lists (default page size is 250)
            var page1Results = extractionsContext.CriteriaListOperations.GetAll();

            //Output
            Status.WriteLine("HasNextPage: {0}", page1Results.HasNextPage);
            if (page1Results.HasNextPage)
            {
                Status.WriteLine("NextLink.Url: {0}", page1Results.NextLink.Url);
            }
            Status.WriteLine("Count: {0}", page1Results.Count());
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "CriteriaList", "GetAll", MethodType.Operation, Publish.Secondary);
            if (page1Results.HasNextPage) //Use a while loop if you need to retrieve all criteria lists
            {
                //Get the second page of instrument lists by passing in the NextLink returned from the previous request
                var page2Results = extractionsContext.CriteriaListOperations.GetAll(page1Results.NextLink);

                //Output
                Status.WriteLine("HasNextPage: {0}", page2Results.HasNextPage);
                if (page2Results.HasNextPage)
                {
                    Status.WriteLine("NextLink.Url: {0}", page2Results.NextLink.Url);
                }
                Status.WriteLine("Count: {0}", page2Results.Count());
            }
            Status.EndNotify(extractionsContext);

            // Delete example's criteria lists
            extractionsContext.CriteriaListOperations.DeleteMultiple(new[] { criteriaList1.ListId, criteriaList2.ListId });
        }

        [Example("GetAvailableFilters for Commodities")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(CriteriaList))]
        public void GetAvailableFiltersForCommodities()
        {
            // Create context
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            // Get available filters for commodities
            Status.Notify(extractionsContext, "CriteriaList", "GetAvailableFilters", MethodType.Operation, Publish.Secondary);
            var availableFilters = extractionsContext.CriteriaListOperations.GetAvailableFilters(CriteriaListType.Commodities);

            // Display some filters for commodities
            var assetStatusFilter = availableFilters.First(s => s.FilterName == "Active Status");
            Status.WriteLine("{0}: Type: {1} Is Universe: {2}", assetStatusFilter.FilterName, assetStatusFilter.FilterType, assetStatusFilter.IsUniverseFilter);

            var assetCategoriesFilter = availableFilters.First(s => s.FilterName == "Asset Categories");
            Status.WriteLine("{0}: Type: {1} Is Universe: {2}", assetCategoriesFilter.FilterName, assetCategoriesFilter.FilterType, assetCategoriesFilter.IsUniverseFilter);
            Status.WriteLine("Available values for {0}:", assetCategoriesFilter.FilterName);
            foreach (var value in assetCategoriesFilter.FilterValues)
            {
                Status.WriteLine("\t{0}: {1}", value.Code, value.Description);
            }
            Status.EndNotify(extractionsContext);
        }

        [Example("GetAvailableFilters for Fixed Income New Issuance")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(CriteriaList))]
        public void GetAvailableFiltersForFixedIncomeNewIssuance()
        {
            // Create context
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            // Get available filters for fixed income new issuance
            Status.Notify(extractionsContext, "CriteriaList", "GetAvailableFilters", MethodType.Operation, Publish.Secondary);
            var availableFilters = extractionsContext.CriteriaListOperations.GetAvailableFilters(CriteriaListType.FixedIncomeNewIssuance);

            // Display some filters for fixed income new issuance
            var couponRateFilter = availableFilters.First(s => s.FilterName == "Coupon Rate");
            Status.WriteLine("{0}: Type: {1} Is Universe: {2}", couponRateFilter.FilterName, couponRateFilter.FilterType, couponRateFilter.IsUniverseFilter);

            var issueDateFilter = availableFilters.First(s => s.FilterName == "Issue Date");
            Status.WriteLine("{0}: Type: {1} Is Universe: {2}", issueDateFilter.FilterName, issueDateFilter.FilterType, issueDateFilter.IsUniverseFilter);

            Status.EndNotify(extractionsContext);
        }

        [Example("GetAvailableFilters for Loans")]
        public void GetAvailableFiltersForLoans()
        {
            // Create context
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            // Get available filters for loans
            Status.Notify(extractionsContext, "CriteriaList", "GetAvailableFilters", MethodType.Operation, Publish.Secondary);
            var availableFilters = extractionsContext.CriteriaListOperations.GetAvailableFilters(CriteriaListType.Loans);

            // Display some filters for loans
            var activeStatusFilter = availableFilters.First(s => s.FilterName == "Active Status");
            Status.WriteLine("{0}: Type: {1} Is Universe: {2}", activeStatusFilter.FilterName, activeStatusFilter.FilterType, activeStatusFilter.IsUniverseFilter);

            var baseRateFilter = availableFilters.First(s => s.FilterName == "Base Rate");
            Status.WriteLine("{0}: Type: {1} Is Universe: {2}", baseRateFilter.FilterName, baseRateFilter.FilterType, baseRateFilter.IsUniverseFilter);
            Status.WriteLine("Available values for {0}:", baseRateFilter.FilterName);
            foreach (var value in baseRateFilter.FilterValues)
            {
                Status.WriteLine("\t{0}: {1}", value.Code, value.Description);
            }
            Status.EndNotify(extractionsContext);
        }

        [Example("GetAvailableFilters for MiFID Sub-Class")]
        public void GetAvailableFiltersForMifidSubclass()
        {
            // Create context
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            // Get available filters for MiFID Sub-Class
            Status.Notify(extractionsContext, "CriteriaList", "GetAvailableFilters", MethodType.Operation, Publish.Secondary);
            var availableFilters = extractionsContext.CriteriaListOperations.GetAvailableFilters(CriteriaListType.MifidSubclass);

            // Display some filters for MiFID Sub-Class
            var assetClassFilter = availableFilters.First(s => s.FilterName == "Asset Class");
            Status.WriteLine("{0}: Type: {1} Is Universe: {2}", assetClassFilter.FilterName, assetClassFilter.FilterType, assetClassFilter.IsUniverseFilter);
            Status.WriteLine("Available values for {0}:", assetClassFilter.FilterName);
            foreach (var value in assetClassFilter.FilterValues)
            {
                Status.WriteLine("\t{0}: {1}", value.Code, value.Description);
            }

            var timeToMaturityBucketFilter = availableFilters.First(s => s.FilterName == "Time to Maturity Bucket");
            Status.WriteLine("{0}: Type: {1} Is Universe: {2}", timeToMaturityBucketFilter.FilterName, timeToMaturityBucketFilter.FilterType, timeToMaturityBucketFilter.IsUniverseFilter);
            Status.WriteLine("Available values for {0}:", timeToMaturityBucketFilter.FilterName);
            foreach (var value in timeToMaturityBucketFilter.FilterValues)
            {
                Status.WriteLine("\t{0}: {1}", value.Code, value.Description);
            }
            Status.EndNotify(extractionsContext);
        }


        [Example("GetAvailableFilters for Publicly-Traded Partnership (PTP)")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(CriteriaList))]
        public void GetAvailableFiltersForPubliclyTradedPartnership()
        {
            // Create context
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            // Get available filters for Publicly-Traded Partnership (PTP)
            Status.Notify(extractionsContext, "CriteriaList", "GetAvailableFilters", MethodType.Operation, Publish.Secondary);
            var availableFilters = extractionsContext.CriteriaListOperations.GetAvailableFilters(CriteriaListType.PubliclyTradedPartnership);

            // Display some filters for Publicly-Traded Partnership (PTP)
            var activeStatusFilter = availableFilters.First(s => s.FilterName == "Active Status");
            Status.WriteLine("{0}: Type: {1} Is Universe: {2}", activeStatusFilter.FilterName, activeStatusFilter.FilterType, activeStatusFilter.IsUniverseFilter);

            var assetTypeFilter = availableFilters.First(s => s.FilterName == "Asset Type");
            Status.WriteLine("{0}: Type: {1} Is Universe: {2}", assetTypeFilter.FilterName, assetTypeFilter.FilterType, assetTypeFilter.IsUniverseFilter);

            Status.EndNotify(extractionsContext);
        }

        [Example("Rename")]
        public void Rename()
        {
            // Create context
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            // Create a criteria list
            var filters = new List<CriteriaListFilter>
            {
                new BooleanFilter {Name = "Active Status", Value = BooleanValue.Yes}
            };
            var criteriaList = extractionsContext.CriteriaListOperations.Create("LoansCriteriaList" + Suffix.New(), CriteriaListType.Loans, filters);
            Status.WriteLine("ListId: {0}", criteriaList.ListId);

            Status.Notify(extractionsContext, "CriteriaList", "Rename", MethodType.Operation, Publish.Secondary);
            // Rename the criteria list
            const string newListName = "RenamedCriteriaList";
            extractionsContext.CriteriaListOperations.Rename(criteriaList, newListName);
            Status.EndNotify(extractionsContext);
            
            Status.Notify(extractionsContext, "CriteriaList", "GetByName", MethodType.Operation, Publish.Secondary);
            // Get list by name
            var renamedList = extractionsContext.CriteriaListOperations.GetByName(newListName);
            Status.EndNotify(extractionsContext);

            // Delete criteria list
            extractionsContext.CriteriaListOperations.Delete(renamedList);
        }

        [Example("AddToInstrumentList")]
        [EndToEndExample(typeof(ExtractionsContext), typeof(CriteriaList))]
        public void AddToInstrumentList()
        {
            // Create context
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, "CriteriaList", "Create", MethodType.Operation, Publish.Secondary);
            // Create criteria list
            var filters = new List<CriteriaListFilter>
            {
                new BooleanFilter { Name = "Active Status", Value = BooleanValue.Yes },
                new BooleanFilter { Name = "Basis Gap Adjusted", Value = BooleanValue.All },
                new MultiValueFilter
                {
                    Name = "Currencies",
                    Op = MultiValueOperator.In,
                    Values = new DssCollection<string>
                    {
                        "USD"
                    }
                },
                new RangeFilter
                {
                    Name = "BTU Per Barrel",
                    Op = RangeOperator.GreaterThanOrEqual,
                    Value = (decimal?) 5.0
                }
            };
            var criteriaList = extractionsContext.CriteriaListOperations.Create("CommoditiesCriteriaList" + Suffix.New(), CriteriaListType.Commodities, filters);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            // Create instrument list
            var instrumentList = new InstrumentList { Name = "ToBeAppended" + Suffix.New() };
            extractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "CriteriaList", "AddToInstrumentList", MethodType.Operation, Publish.Secondary);
            // Add criteria lists instruments to instrument list and a new instrument list
            var newListName = "ToBeCreated" + Suffix.New();
            var appendResults = extractionsContext.CriteriaListOperations.AddToInstrumentList(criteriaList, new List<string> { instrumentList.Name, newListName });
            foreach (var result in appendResults)
            {
                Status.WriteLine("{0}", result.Message);
                Status.WriteLine("Added {0} successfully to list", result.AppendedInstrumentCount);
                Status.WriteLine("Detected {0} duplicates", result.DuplicateInstrumentCount);
            }
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Get");
            // Get instrument list to check instrument count
            var appendedList = extractionsContext.InstrumentListOperations.Get(instrumentList.ListId);
            Status.WriteLine("List count: {0}", appendedList.Count);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "GetByName");
            // Get new instrument list by name
            var createdList = extractionsContext.InstrumentListOperations.GetByName(newListName);
            Status.WriteLine("List count: {0}", createdList.Count);
            Status.EndNotify(extractionsContext);

            // Delete criteria list
            extractionsContext.CriteriaListOperations.Delete(criteriaList);
            
            // Delete instrument lists
            extractionsContext.InstrumentListOperations.DeleteMultiple(new[] { appendedList.ListId, createdList.ListId });
        }
    }
}
