﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Security.Policy;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using Dss.Api.Examples.Annotations;

namespace Dss.Api.Examples.Internal
{
    public class Example : INotifyPropertyChanged
    {
        private readonly MethodInfo _methodInfo;
        private readonly List<ExampleStep> _steps;

        #region Constructor

        public Example(MethodInfo methodInfo)
        {
            _steps = new List<ExampleStep>();

            ShowPass = Visibility.Collapsed;
            ShowFail = Visibility.Collapsed;

            _methodInfo = methodInfo;
            var classType = _methodInfo.DeclaringType;
            if (classType == null)
                return;
            //Get the method Name
            MethodName = String.Format("{0}.{1}", classType.FullName.Replace("Dss.Api.Examples.", ""), _methodInfo.Name);

            //Get the Description
            var exampleAttribute = _methodInfo.GetCustomAttributes(typeof(ExampleAttribute), false).OfType<ExampleAttribute>().FirstOrDefault();
            if (exampleAttribute == null)
                return;
            Description = exampleAttribute.Description;

            //Get the example class description
            var classExampleAttribute = classType
                .GetCustomAttributes(typeof(ExampleAttribute), false)
                .OfType<ExampleAttribute>()
                .FirstOrDefault();
            if (classExampleAttribute == null)
                return;
            ClassDescription = ExampleListings.ScrubDescription(classExampleAttribute.Description);

            //End-to-end example information
            var endToEndExampleAttributes = _methodInfo
                .GetCustomAttributes(typeof(EndToEndExampleAttribute), false)
                .OfType<EndToEndExampleAttribute>().ToArray();
            if (endToEndExampleAttributes.Any())
            {
                EndToEndExamples = endToEndExampleAttributes.Select(a => new EndToEndExample
                {
                    IsKeyMechanism = a.IsKeyMechanism,
                    ContextType = a.ContextType,
                    EntityType = a.EntityType
                })
                .ToArray();
            }
        }

        #endregion

        #region Name / Description / etc.

        public string MethodName { get; private set; }

        public string ClassDescription { get; private set; }

        public string Description { get; private set; }

        public string Name
        {
            get { return String.Format("{0}: {1}", ClassDescription, Description); }
        }

        #endregion

        #region Steps

        public IReadOnlyCollection<ExampleStep> Steps { get { return _steps.AsReadOnly(); } }

        public void ClearSteps()
        {
            lock (_steps)
            {
                _steps.Clear();
                UpdatePassFail();
            }
        }

        public void AddStep(ExampleStep step)
        {
            lock (_steps)
            {
                _steps.Add(step);
                UpdatePassFail();
            }
        }

        private void UpdatePassFail()
        {
            var values = Steps.ToArray();
            if (!values.Any())
            {
                ShowPass = Visibility.Collapsed;
                ShowFail = Visibility.Collapsed;
                return;
            }
            var fail = values.Any(s => s.Result != null && s.Result.Error != null);
            ShowPass = fail ? Visibility.Collapsed : Visibility.Visible;
            ShowFail = fail ? Visibility.Visible : Visibility.Collapsed;
        }

        #endregion

        #region Pass / Fail

        private Visibility _showPass = Visibility.Collapsed;
        public Visibility ShowPass 
        { 
            get
            {
                return _showPass;
            } 
            private set
            {
                _showPass = value;
                OnPropertyChanged();
            }
        }

        private Visibility _showFail = Visibility.Collapsed;
        public Visibility ShowFail
        {
            get
            {
                return _showFail;
            }
            private set
            {
                _showFail = value;
                OnPropertyChanged();
            }
        }

        #endregion

        public IEnumerable<EndToEndExample> EndToEndExamples { get; private set; }

        #region Run

        public Task<bool> Run(StreamWriter streamWriter, CancellationToken cancellationToken = default(CancellationToken))
        {
            ClearSteps();
            Status.ResetStepNumber();
            var exampleSucceeded = false;

            return Task.Run(() =>
            {
                try
                {
                    var exampleClass = Activator.CreateInstance(_methodInfo.DeclaringType);
                    
                    Status.RunningExample = this;

                    //Run the example
                    _methodInfo.Invoke(exampleClass, new object[] {});

                    LogResults(streamWriter, string.Format("Example Passed: {0}", Name));

                    UpdatePassFail();

                    exampleSucceeded = true;
                }
                catch (TargetInvocationException ex)
                {
                    Status.Error(ex.InnerException);

                    LogResults(streamWriter, string.Format("Example Failed: {0}, Exception = {1}", Name, ex.InnerException.Message));
                }
                catch (Exception ex)
                {
                    Status.Error(ex.InnerException);

                    LogResults(streamWriter, string.Format("Example Failed: {0}, Exception = {1}", Name, ex.InnerException.Message));
                }
                finally
                {
                    Status.RunningExample = null;
                }

                return exampleSucceeded;

            }, cancellationToken);

        }

        #endregion

        private static void LogResults(TextWriter textWriter, string message)
        {
            if (textWriter != null)
            {
                textWriter.WriteLine(DateTime.Now + " - " + message);
                textWriter.Flush();
            }
        }

        public override string ToString()
        {
            return Description;
        }

        public CodeSnippet GetFullCSharpCode()
        {
            var extractor = new ExampleExtractor(null, null, null, null, false, false);

            return extractor.GetCSharpFull(this);
        }

        public event PropertyChangedEventHandler PropertyChanged;

        [NotifyPropertyChangedInvocator]
        protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            var handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
        }

    }

    public class EndToEndExample
    {
        public bool IsKeyMechanism { get; set; }
        public Type ContextType { get; set; }
        public Type EntityType { get; set; }
    }
}
