﻿using System;
using System.IO;
using System.Linq;
using System.Threading;
using Dss.Api.Examples.Internal;
using Dss.Api.Examples.Properties;
using DataScope.Select.Api;
using DataScope.Select.Api.Authentication;
using DataScope.Select.Api.Content;
using DataScope.Select.Api.Core;
using DataScope.Select.Api.Core.HttpOData;
using DataScope.Select.Api.Extractions;
using DataScope.Select.Api.Extractions.ExtractionRequests;
using DataScope.Select.Api.Extractions.ReportExtractions;
using DataScope.Select.Api.Extractions.ReportTemplates;
using DataScope.Select.Api.Extractions.Schedules;
using DataScope.Select.Api.Extractions.SubjectLists;
using DataScope.Select.Api.Jobs;
using DataScope.Select.Api.Users;

namespace Dss.Api.Examples.KeyMechanisms
{
    [Example("2. Key Mechanisms")]
    public class KeyMechanismsExamples
    {
        #region Authenticate

        //DO NOT CHANGE THE EXAMPLE NAME, the name is used verbatim in the Help documentation site.
        [Example("Authenticate: Primary Example")]
        [EndToEndExample(typeof(AuthenticationContext))]
        [EndToEndExample]
        public void Authentication()
        {
            ContextHelper.ClearToken();
            var usersContext = ContextHelper.CreateUsersContext(false);
            Status.Notify(usersContext, null, "RequestToken");
            //Store the session token for use on other contexts.  Storing and re-using the session token
            //will avoid unnecessary authentication requests and improve response times.
            var sessionToken = usersContext.SessionToken;

            //Output
            Status.WriteLine(sessionToken);
            Status.EndNotify(usersContext);

            Status.Notify(usersContext, "User", "Get");
            //Fetch the user information using the current token applied to the header.
            usersContext.UserOperations.Get(int.Parse(Settings.Default.UserId));
            Status.EndNotify(usersContext);
        }

        #endregion

        #region Lazy Loading

        //DO NOT CHANGE THE EXAMPLE NAME, the name is used verbatim in the Help documentation site.
        [Example("Associations: Lazy Loading")]
        [EndToEndExample(typeof(UsersContext))]
        [EndToEndExample]
        public void LazyLoadUserPreferences()
        {
            /* Associated entities can be lazy loaded by submitting a second request.  Users of the .Net toolkit can users
             * LoadProperty while Direct HTTP users can issue the request with the Association name suffixed to the entity url. */
            var usersContext = ContextHelper.CreateUsersContext();
            Status.Notify(usersContext, "User", "GetAll");
            //Fetch the user (parent entity)
            var user = usersContext.UserOperations.Get(int.Parse(Settings.Default.UserId));
            Status.EndNotify(usersContext);

            Status.Notify(usersContext, "User", "Preferences");
            //Get the preferences by loading the User.UserPreferences.  Until this method is called, User.UserPreferences is null.
            usersContext.LoadProperty(user, u => u.Preferences);

            Status.EndNotify(usersContext);
        }

        #endregion

        #region Paging

        //DO NOT CHANGE THE EXAMPLE NAME, the name is used verbatim in the Help documentation site.
        [Example("Paging: Primary Example")]
        [EndToEndExample]
        public void Paging()
        {
            /*
             * Server-side paging returns a limited number of results on the first request (typically 250 rows).  The returned 
             * results include a next link (url) which will return the second page of data.  The DSS Toolkit provides a custom
             * IDssEnumerable and IDssQueryable which have HasNextPage and NextLink properties.
             * 
             * You can request a smaller or larger page size which the server may or may not honor. */

            var extractionsContext = ContextHelper.CreateExtractionsContext();
            //Set the page size to 1 record per page for the purposes of this example
            extractionsContext.Preferences.MaxPageSize = 1;

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            //Create an instrument list 
            extractionsContext.InstrumentListOperations.Create(new InstrumentList { Name = "Instrument List Name A" + Suffix.New() });
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            //Create another instrument list 
            extractionsContext.InstrumentListOperations.Create(new InstrumentList { Name = "Instrument List Name B" + Suffix.New() });
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "GetAll");
            //Get the first page of instrument lists
            var pagedResults = extractionsContext.InstrumentListOperations.GetAll();

            //Output
            Status.WriteLine("HasNextPage: {0}", pagedResults.HasNextPage);
            Status.WriteLine("NextLink.Url: {0}", pagedResults.NextLink.Url);
            Status.WriteLine("Count: {0}", pagedResults.Count());
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "GetAll");
            if (pagedResults.HasNextPage) //This would more than likely be a while loop if you wanted to retrieve all pages.
            {
                //Get the second page of instrument lists by passing in the NextLink returned from the previous request
                pagedResults = extractionsContext.InstrumentListOperations.GetAll(pagedResults.NextLink);

                //Output
                Status.WriteLine("HasNextPage: {0}", pagedResults.HasNextPage);

                if (pagedResults.HasNextPage)
                {
                    Status.WriteLine("NextLink.Url: {0}", pagedResults.NextLink.Url);
                }

                Status.WriteLine("Count: {0}", pagedResults.Count());
            }
            Status.EndNotify(extractionsContext);
        }

        //DO NOT CHANGE THE EXAMPLE NAME, the name is used verbatim in the Help documentation site.
        [Example("Paging: Retrieving all pages")]
        [EndToEndExample]
        public void PagingAllPages()
        {
            /* Server-side paging returns a limited number of results on the first request (typically 250 rows).  The returned 
             * results include a next link (url) which will return the second page of data.  The DSS Toolkit provides a custom
             * IDssEnumerable and IDssQueryable which have HasNextPage and NextLink properties.
             * You can request a smaller or larger page size which the server may or may not honor. */

            var extractionsContext = ContextHelper.CreateExtractionsContext();
            //Set the page size to 1 record per page for the purposes of this example
            extractionsContext.Preferences.MaxPageSize = 1;

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            //Create an instrument list 
            extractionsContext.InstrumentListOperations.Create(new InstrumentList { Name = "Instrument List Name A" + Suffix.New() });
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            //Create another instrument list 
            extractionsContext.InstrumentListOperations.Create(new InstrumentList { Name = "Instrument List Name B" + Suffix.New() });
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "GetAll");
            //Get the first page of instrument lists
            var pagedResults = extractionsContext.InstrumentListOperations.GetAll();
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "GetAll");
            while (pagedResults.HasNextPage)
            {
                pagedResults = extractionsContext.InstrumentListOperations.GetAll(pagedResults.NextLink);
                Status.WriteLine("Count: {0}", pagedResults.Count());
            }
            Status.EndNotify(extractionsContext);
        }

        #endregion

        #region Async

        //DO NOT CHANGE THE EXAMPLE NAME, the name is used verbatim in the Help documentation site.
        [Example("Async: Primary Example")]
        [EndToEndExample]
        public void Async()
        {
            /* Asynchronous requests provides a mechanism for supporting long running requests with status updates and progress
             * messages while circumventing the need to make firewall changes.
             * Many of our long-running requests support async.  For example Schedule.ExtractNow or Extract (on demand).  If async 
             * is enabled (it is enabled by default for toolkit users, but not for HTTP direct users.  Asynchronous responses can 
             * be enabled by Prefer: respond-async in the header.  If respond-async is not included in the header, the request 
             * will be handled synchronously and may fail if the request takes more than 30 minutes due to firewall timeouts.
             * If the asynchronous request can be responded to within the poll time, the request is executed synchronously.  If
             * the request takes longer than the poll time, a 202 (accepted) is returned to the client with a monitor location
             * URL.  Once the results are available the payload will be returned on the next request to the monitor location.
             * 
             * .Net SDK users do not need to be concerned with this since the SDK will default to asynchronous requests and 
             * automatically poll for the response.  The effect is that the behavior appears to be the same as synchronous 
             * request.  See "Async: Starting a Job" for examples on how you can be take control of the polling process.
             * 
             * This example demonstrates the asynchronous behavior (which is the standard behavior for SDK users).
             */

            var extractionsContext = ContextHelper.CreateExtractionsContext();
            Status.Notify(extractionsContext, null, "Extract");
            //Reduce the poll time for the purposes of the example
            extractionsContext.Preferences.WaitSeconds = 5;
            //Create the request
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                IdentifierList = SubjectIdentifierList.CreateInstrumentIdentifierList(new[]
                    {
                       SubjectIdentifier.CreateInstrumentIdentifier("IBM.N", IdentifierType.Ric, null, null, null, null)
                    }, null, false),
                ContentFieldNames = new[] { "Ask Price", "Asset Category", "Asset Category Description", "Asset ID", "Asset Status",
                "Asset Status Description", "Asset SubType", "Asset SubType Description", "Asset Type",
                "Asset Type Description", "Bid Price", "Block Trades" }
            };

            //Extract
            var extractedRows = extractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var field in extractedRows.First().DynamicProperties)
                    Status.WriteLine(field.Key + ": " + field.Value);
            }
            Status.EndNotify(extractionsContext);
        }

        //DO NOT CHANGE THE EXAMPLE NAME, the name is used verbatim in the Help documentation site.
        [Example("Async: Synchronous Override")]
        [EndToEndExample]
        public void AsyncSynchronousOverride()
        {
            /* Asynchronous requests provides a mechanism for supporting long running requests with status updates and progress
             * messages while circumventing the need to make firewall changes.
             * Many of our long-running requests support async.  For example Schedule.ExtractNow or Extract (on demand).  If async 
             * is enabled (it is enabled by default for toolkit users, but not for HTTP direct users.  Asynchronous responses can 
             * be enabled by Prefer: respond-async in the header.  If respond-async is not included in the header, the request 
             * will be handled synchronously and may fail if the request takes more than 30 minutes due to firewall timeouts.
             * If the asynchronous request can be responded to within the poll time, the request is executed synchronously.  If
             * the request takes longer than the poll time, a 202 (accepted) is returned to the client with a monitor location
             * URL.  Once the results are available the payload will be returned on the next request to the monitor location.
             * 
             * This example demonstrates the synchronous behavior.  
             * 
             * The ability to invoke asynchronous methods synchronously is only provided for users who are using primitive 
             * clients and whose extraction requests are completed in less than 30 minutes.
             * 
             * THIS IS NOT THE RECOMMENDED APPROACH FOR .Net SDK OR HTTP DIRECT USERS.
             */

            var extractionsContext = ContextHelper.CreateExtractionsContext();
            Status.Notify(extractionsContext, null, "Extract");
            //Reduce the poll time for the purposes of the example
            extractionsContext.Preferences.RespondAsync = false;
            //Create the request
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                IdentifierList = SubjectIdentifierList.CreateInstrumentIdentifierList(new[]
                    {
                       SubjectIdentifier.CreateInstrumentIdentifier("IBM.N", IdentifierType.Ric, null, null, null, null)
                    }, null, false),
                ContentFieldNames = new[] { "Ask Price", "Asset Category", "Asset Category Description", "Asset ID", "Asset Status",
                "Asset Status Description", "Asset SubType", "Asset SubType Description", "Asset Type",
                "Asset Type Description", "Bid Price", "Block Trades" }
            };

            //Extract
            var extractedRows = extractionsContext.Extract(extractionRequest).ToArray();

            //Output
            if (!extractedRows.Any())
                Status.WriteLine("No rows returned");
            else
            {
                foreach (var field in extractedRows.First().DynamicProperties)
                    Status.WriteLine(field.Key + ": " + field.Value);
            }
            Status.EndNotify(extractionsContext);
        }

        [Example("Async: Starting a Job")]
        [EndToEndExample]
        public void AsyncJobStart()
        {
            /* Asynchronous requests provides a mechanism for supporting long running requests with status updates and progress
             * messages while circumventing the need to make firewall changes.
             * Many of our long-running requests support async.  For example Schedule.ExtractNow or Extract (on demand).  If async 
             * is enabled (it is enabled by default for toolkit users, but not for HTTP direct users.  Asynchronous responses can 
             * be enabled by Prefer: respond-async in the header.  If respond-async is not included in the header, the request 
             * will be handled synchronously and may fail if the request takes more than 30 minutes due to firewall timeouts.
             * If the asynchronous request can be responded to within the poll time, the request is executed synchronously.  If
             * the request takes longer than the poll time, a 202 (accepted) is returned to the client with a monitor location
             * URL.  Once the results are available the payload will be returned on the next request to the monitor location.
             * 
             * The SDK provides a means for you to take control of the polling process which can be useful for very long running 
             * requests where you may want to cancel the extraction or retrieve the results in a different process or after a 
             * process restart.  All async methods offer a method with a Start suffix (e.g. context.ExtractWithNotesStart) which 
             * immediately returns control to the caller and returns a handle for the job.  This job handle can be converted to 
             * a string (ToString()) and recreated from this string.  This allows the handle to be persisted across client sessions.
             * 
             * This example demonstrates how to start a request, use the job handle to check the status and use the
             * job handle to retrieve the results.
             */
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the request
            //TODO: Convert this to TRTH when templates are published
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "191216100", IdentifierType = IdentifierType.Cusip },
                    new InstrumentIdentifier { Identifier = "2005973", IdentifierType = IdentifierType.Sedol },
                    new InstrumentIdentifier { Identifier = "AAPL.OQ", IdentifierType = IdentifierType.Ric }
                }, null, false),
                ContentFieldNames = new[] { "Asset Status", "Asset Type", "Bid Price" }
            };

            //Extract using the "Start" method.  Start methods return control to the caller without polling the 
            //monitor url until results are returned.  This may be useful if the job is a very long running job
            //as the case may be with Tick History content depending on the conditions/criteria.  The job can
            //also be persisted to string and re-hydrated.  See the Async help page for additional information.
            var job = extractionsContext.ExtractWithNotesStart(extractionRequest);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, null, "MonitorJob", MethodType.Operation, Publish.Primary);
            while (job != null && job.Status != JobStatus.Completed && job.Status != JobStatus.Error
                   && job.Status != JobStatus.Cancelled && job.Status != JobStatus.Purged)
            {
                //Retrieve an updated status of the job
                job = extractionsContext.MonitorJob(job);
            }
            //Output
            if (job != null && job.Result != null && job.Status == JobStatus.Completed)
            {
                Status.WriteLine(job.Result.Notes);
                foreach (var extractRow in job.Result.Contents)
                    Status.WriteLine(extractRow.Identifier + ":" + extractRow.IdentifierType);
            }
            Status.EndNotify(extractionsContext);
        }

        [Example("Async: Cancelling a Job")]
        [EndToEndExample]
        public void AsyncJobCancel()
        {
            /* Asynchronous requests provides a mechanism for supporting long running requests with status updates and progress
             * messages while circumventing the need to make firewall changes.
             * Many of our long-running requests support async.  For example Schedule.ExtractNow or Extract (on demand).  If async 
             * is enabled (it is enabled by default for toolkit users, but not for HTTP direct users.  Asynchronous responses can 
             * be enabled by Prefer: respond-async in the header.  If respond-async is not included in the header, the request 
             * will be handled synchronously and may fail if the request takes more than 30 minutes due to firewall timeouts.
             * If the asynchronous request can be responded to within the poll time, the request is executed synchronously.  If
             * the request takes longer than the poll time, a 202 (accepted) is returned to the client with a monitor location
             * URL.  Once the results are available the payload will be returned on the next request to the monitor location.
             * 
             * The SDK provides a means for you to take control of the polling process which can be useful for very long running 
             * requests where you may want to cancel the extraction or retrieve the results in a different process or after a 
             * process restart.  All async methods offer a method with a Start suffix (e.g. context.ExtractWithNotesStart) which 
             * immediately returns control to the caller and returns a handle for the job.  This job handle can be converted to 
             * a string (ToString()) and recreated from this string.  This allows the handle to be persisted across client sessions.
             * 
             * This example demonstrates how to start a request, cancel the request and check on it's status.
             */
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the request
            var extractionRequest = new TickHistoryIntradaySummariesExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "191216100", IdentifierType = IdentifierType.Cusip },
                    new InstrumentIdentifier { Identifier = "2005973", IdentifierType = IdentifierType.Sedol },
                    new InstrumentIdentifier { Identifier = "AAPL.OQ", IdentifierType = IdentifierType.Ric }
                }, null, false),
                ContentFieldNames = new[] { "Close Ask", "Close Bid", "High" },
                Condition = new TickHistoryIntradaySummariesCondition
                {
                    DaysAgo = 5,
                    MessageTimeStampIn = TickHistoryTimeOptions.LocalExchangeTime,
                    ReportDateRangeType = ReportDateRangeType.Delta,
                    ExtractBy = TickHistoryExtractByMode.Ric,
                    SortBy = TickHistorySort.SingleByRic,
                    Preview = PreviewMode.None,
                    SummaryInterval = TickHistorySummaryInterval.FifteenMinutes,
                }
            };

            //Start the job
            var job = extractionsContext.ExtractRawStart(extractionRequest);
            Status.EndNotify(extractionsContext);

            Thread.Sleep(3000);

            Status.Notify(extractionsContext, null, "CancelJob", MethodType.Operation, Publish.Primary);
            //Attempt cancellation of the job
            extractionsContext.CancelJob(job);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, null, "MonitorJob", MethodType.Operation, Publish.Secondary);
            //Retrieve an updated status of the job.  Note: It may take a few seconds, perhaps a minute for larger
            //extractions for the cancellation to take effect. It is also possible that the cancellation may not
            //be accepted.
            try
            {
                job = extractionsContext.MonitorJob(job);
            }
            catch (NotFoundException)
            {
                Status.WriteLine("Status: {NotFound}");
            }

            Status.EndNotify(extractionsContext);
            throw new InvalidOperationException("Expected NotFoundException but no exception was thrown.");
        }

        [Example("Async: Persisting a Job Handle")]
        [EndToEndExample]
        public void AsyncJobPersistHandle()
        {
            /* Asynchronous requests provides a mechanism for supporting long running requests with status updates and progress
             * messages while circumventing the need to make firewall changes.
             * Many of our long-running requests support async.  For example Schedule.ExtractNow or Extract (on demand).  If async 
             * is enabled (it is enabled by default for toolkit users, but not for HTTP direct users.  Asynchronous responses can 
             * be enabled by Prefer: respond-async in the header.  If respond-async is not included in the header, the request 
             * will be handled synchronously and may fail if the request takes more than 30 minutes due to firewall timeouts.
             * If the asynchronous request can be responded to within the poll time, the request is executed synchronously.  If
             * the request takes longer than the poll time, a 202 (accepted) is returned to the client with a monitor location
             * URL.  Once the results are available the payload will be returned on the next request to the monitor location.
             * 
             * The SDK provides a means for you to take control of the polling process which can be useful for very long running 
             * requests where you may want to cancel the extraction or retrieve the results in a different process or after a 
             * process restart.  All async methods offer a method with a Start suffix (e.g. context.ExtractWithNotesStart) which 
             * immediately returns control to the caller and returns a handle for the job.  This job handle can be converted to 
             * a string (ToString()) and recreated from this string.  This allows the handle to be persisted across client sessions.
             * 
             * This example demonstrates how to persist the job handle and recreate the job.
             */
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the request
            //TODO: Convert this to TRTH when templates are published
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "191216100", IdentifierType = IdentifierType.Cusip },
                    new InstrumentIdentifier { Identifier = "2005973", IdentifierType = IdentifierType.Sedol },
                    new InstrumentIdentifier { Identifier = "AAPL.OQ", IdentifierType = IdentifierType.Ric }
                }, null, false),
                ContentFieldNames = new[] { "Asset Status", "Asset Type", "Bid Price" }
            };

            //Start the job
            var job = extractionsContext.ExtractWithNotesStart(extractionRequest);
            //Get a handle for the job which can be persisted and used at any time (up to 7 days after the extraction request)
            //to retrieve the status and/or results.
            var handle = job.ToString();
            Status.WriteLine("Job Handle: {0}", handle);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, null, "MonitorJob", MethodType.Operation, Publish.Secondary);
            //Create an AsyncJob from the handle
            job = AsyncJob<ExtractionResult>.Parse(handle);
            //Retrieve an updated status of the job
            job = extractionsContext.MonitorJob(job);
            Status.WriteLine("Status: {0}", job.Status);
            Status.EndNotify(extractionsContext);
        }

        [Example("Async: Get Active Jobs")]
        [EndToEndExample]
        public void AsyncJobGetActive()
        {
            /* Asynchronous requests provides a mechanism for supporting long running requests with status updates and progress
             * messages while circumventing the need to make firewall changes.
             * Many of our long-running requests support async.  For example Schedule.ExtractNow or Extract (on demand).  If async 
             * is enabled (it is enabled by default for toolkit users, but not for HTTP direct users.  Asynchronous responses can 
             * be enabled by Prefer: respond-async in the header.  If respond-async is not included in the header, the request 
             * will be handled synchronously and may fail if the request takes more than 30 minutes due to firewall timeouts.
             * If the asynchronous request can be responded to within the poll time, the request is executed synchronously.  If
             * the request takes longer than the poll time, a 202 (accepted) is returned to the client with a monitor location
             * URL.  Once the results are available the payload will be returned on the next request to the monitor location.
             * 
             * There are two basic mechanisms for managing your active jobs (asynchronous requests).  The first way is to 
             * persist the job handle, or in the case of HTTP direct requests you can persist the monitor url.  This allows you 
             * to fetch the job status and results using the job handle / monitor url.  The second approach is to use the Jobs
             * resource which allows you to retrieve all active or completed jobs.  The returned list of jobs include their 
             * status and the monitor URL.  The SDK uses this information to create an instance of an IAsyncJob.  For HTTP 
             * direct users, the monitor URL is all that is required to retrieve the results or subsequent status updates.
             * 
             * The Job resource on the Jobs Context can be queries as a regular resource (e.g. /jobs/jobs).  There is a 
             * method for retrieving only active (in flight jobs) /jobs/jobGetActive and another for retrieving 
             * completed jobs /jobs/jobGetCompleted.
             * 
             * This example demonstrates how to retrieve your active jobs.
             */
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the request
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "191216100", IdentifierType = IdentifierType.Cusip },
                    new InstrumentIdentifier { Identifier = "2005973", IdentifierType = IdentifierType.Sedol },
                    new InstrumentIdentifier { Identifier = "AAPL.OQ", IdentifierType = IdentifierType.Ric }
                }, null, false),
                ContentFieldNames = new[] { "Asset Status", "Asset Type", "Bid Price" }
            };

            //Start the job
            var job = extractionsContext.ExtractWithNotesStart(extractionRequest);
            Status.WriteLine("Result Monitor Uri: {0}", job.MonitorUri);
            Status.EndNotify(extractionsContext);

            var jobsContext = ContextHelper.CreateJobsContext();
            jobsContext.Preferences.MaxPageSize = 3;
            Status.Notify(jobsContext, "Job", "GetActive", MethodType.Operation, Publish.Primary);
            var activeJobs = jobsContext.JobOperations.GetActive();
            //Output the active jobs
            foreach (var activeJob in activeJobs.Take(10))
                Status.WriteLine("job: id {0}, status {1}, progress {2}, monitor url {3}",
                    activeJob.JobId, activeJob.Status, activeJob.ProgressPercentage, activeJob.MonitorUrl);
            Status.EndNotify(jobsContext);

            Status.Notify(jobsContext, "Job", "GetActive");
            //If there is another page of results, fetch the next page.  This is just an example, in reality if you
            //were interested in the all the pages you would just loop until completedJobs.HasNextPage is false.
            if (activeJobs.HasNextPage)
            {
                activeJobs = jobsContext.JobOperations.GetActive(activeJobs.NextLink);
                //Output the active jobs
                foreach (var completedJob in activeJobs.Take(10))
                    Status.WriteLine("job: id {0}, status {1}, progress {2}, monitor url {3}",
                        completedJob.JobId, completedJob.Status, completedJob.ProgressPercentage,
                        completedJob.MonitorUrl);
            }
            Status.EndNotify(jobsContext);
        }

        [Example("Async: Get Completed Jobs")]
        [EndToEndExample]
        public void AsyncJobGetCompleted()
        {
            /* Asynchronous requests provides a mechanism for supporting long running requests with status updates and progress
             * messages while circumventing the need to make firewall changes.
             * Many of our long-running requests support async.  For example Schedule.ExtractNow or Extract (on demand).  If async 
             * is enabled (it is enabled by default for toolkit users, but not for HTTP direct users.  Asynchronous responses can 
             * be enabled by Prefer: respond-async in the header.  If respond-async is not included in the header, the request 
             * will be handled synchronously and may fail if the request takes more than 30 minutes due to firewall timeouts.
             * If the asynchronous request can be responded to within the poll time, the request is executed synchronously.  If
             * the request takes longer than the poll time, a 202 (accepted) is returned to the client with a monitor location
             * URL.  Once the results are available the payload will be returned on the next request to the monitor location.
             * 
             * There are two basic mechanisms for managing your active jobs (asynchronous requests).  The first way is to 
             * persist the job handle, or in the case of HTTP direct requests you can persist the monitor url.  This allows you 
             * to fetch the job status and results using the job handle / monitor url.  The second approach is to use the Jobs
             * resource which allows you to retrieve all active or completed jobs.  The returned list of jobs include their 
             * status and the monitor URL.  The SDK uses this information to create an instance of an IAsyncJob.  For HTTP 
             * direct users, the monitor URL is all that is required to retrieve the results or subsequent status updates.
             * 
             * The Job resource on the Jobs Context can be queries as a regular resource (e.g. /jobs/jobs).  There is a 
             * method for retrieving only active (in flight jobs) /jobs/jobGetActive and another for retrieving 
             * completed jobs /jobs/jobGetCompleted.
             * 
             * This example demonstrates how to retrieve your completed jobs.
             */
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create the request
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "191216100", IdentifierType = IdentifierType.Cusip },
                    new InstrumentIdentifier { Identifier = "2005973", IdentifierType = IdentifierType.Sedol },
                    new InstrumentIdentifier { Identifier = "AAPL.OQ", IdentifierType = IdentifierType.Ric }
                }, null, false),
                ContentFieldNames = new[] { "Asset Status", "Asset Type", "Bid Price" }
            };

            //Start the job
            var job = extractionsContext.ExtractWithNotesStart(extractionRequest);
            Status.WriteLine("Result Monitor Uri: {0}", job.MonitorUri);
            Status.EndNotify(extractionsContext);

            var jobsContext = ContextHelper.CreateJobsContext();
            jobsContext.Preferences.MaxPageSize = 3;

            Status.Notify(jobsContext, "Job", "GetCompleted", MethodType.Operation, Publish.Primary);

            //Get the first page of completed jobs
            var completedJobs = jobsContext.JobOperations.GetCompleted();
            //Output the active jobs
            foreach (var completedJob in completedJobs.Take(10))
                Status.WriteLine("job: id {0}, status {1}, progress {2}, monitor url {3}",
                    completedJob.JobId, completedJob.Status, completedJob.ProgressPercentage, completedJob.MonitorUrl);
            Status.EndNotify(jobsContext);

            Status.Notify(jobsContext, "Job", "GetCompleted");
            //If there is another page of results, fetch the next page.  This is just an example, in reality if you
            //were interested in the all the pages you would just loop until completedJobs.HasNextPage is false.
            if (completedJobs.HasNextPage)
            {
                completedJobs = jobsContext.JobOperations.GetCompleted(completedJobs.NextLink);
                //Output the active jobs
                foreach (var completedJob in completedJobs.Take(10))
                    Status.WriteLine("job: id {0}, status {1}, progress {2}, monitor url {3}",
                        completedJob.JobId, completedJob.Status, completedJob.ProgressPercentage,
                        completedJob.MonitorUrl);
            }
            Status.EndNotify(jobsContext);
        }

        [Example("Async: Get Active Jobs and Monitor")]
        [EndToEndExample]
        public void AsyncJobGetActiveAndMonitor()
        {
            /* Asynchronous requests provides a mechanism for supporting long running requests with status updates and progress
             * messages while circumventing the need to make firewall changes.
             * Many of our long-running requests support async.  For example Schedule.ExtractNow or Extract (on demand).  If async 
             * is enabled (it is enabled by default for toolkit users, but not for HTTP direct users.  Asynchronous responses can 
             * be enabled by Prefer: respond-async in the header.  If respond-async is not included in the header, the request 
             * will be handled synchronously and may fail if the request takes more than 30 minutes due to firewall timeouts.
             * If the asynchronous request can be responded to within the poll time, the request is executed synchronously.  If
             * the request takes longer than the poll time, a 202 (accepted) is returned to the client with a monitor location
             * URL.  Once the results are available the payload will be returned on the next request to the monitor location.
             * 
             * There are two basic mechanisms for managing your active jobs (asynchronous requests).  The first way is to 
             * persist the job handle, or in the case of HTTP direct requests you can persist the monitor url.  This allows you 
             * to fetch the job status and results using the job handle / monitor url.  The second approach is to use the Jobs
             * resource which allows you to retrieve all active or completed jobs.  The returned list of jobs include their 
             * status and the monitor URL.  The SDK uses this information to create an instance of an IAsyncJob.  For HTTP 
             * direct users, the monitor URL is all that is required to retrieve the results or subsequent status updates.
             * 
             * The Job resource on the Jobs Context can be queries as a regular resource (e.g. /jobs/jobs).  There is a 
             * method for retrieving only active (in flight jobs) /jobs/jobGetActive and another for retrieving 
             * completed jobs /jobs/jobGetCompleted.
             * 
             * This example demonstrates how to retrieve your active jobs, monitor the status and retrieve the results.  This 
             * scenario is different because the job handle is not available, but rather all active jobs are queried and 
             * subsequently monitored.  The preferred workflow is to retain the job handle for each created job and use that 
             * to examine the status and retrieve the results.  The disadvantage to this approach is that the strongly-typed
             * AsyncJob is lost which will require casting of the results.  Of course this does not matter at all to HTTP
             * direct users.
             */
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            Status.Notify(extractionsContext, null, "ExtractWithNotes", MethodType.Operation, Publish.Secondary);
            //Create an extraction request so that there will be an in-process job for the example.
            var extractionRequest = new EndOfDayPricingExtractionRequest
            {
                IdentifierList = InstrumentIdentifierList.Create(
                new[]
                {
                    new InstrumentIdentifier { Identifier = "191216100", IdentifierType = IdentifierType.Cusip },
                    new InstrumentIdentifier { Identifier = "2005973", IdentifierType = IdentifierType.Sedol },
                    new InstrumentIdentifier { Identifier = "AAPL.OQ", IdentifierType = IdentifierType.Ric }
                }, null, false),
                ContentFieldNames = new[] { "Asset Status", "Asset Type", "Bid Price" }
            };

            //Start the job
            var job = extractionsContext.ExtractWithNotesStart(extractionRequest);
            Status.WriteLine("Result Monitor Uri: {0}", job.MonitorUri);
            Status.EndNotify(extractionsContext);

            var jobsContext = ContextHelper.CreateJobsContext();
            jobsContext.Preferences.MaxPageSize = 3;
            //Get the active jobs.  This will return all in-flight jobs along with their monitor url.
            Status.Notify(jobsContext, "Job", "GetActive");
            var activeJobs = jobsContext.JobOperations.GetActive();
            //Output the active jobs
            foreach (var activeJob in activeJobs.Take(10))
                Status.WriteLine("job: id {0}, status {1}, progress {2}, monitor url {3}",
                    activeJob.JobId, activeJob.Status, activeJob.ProgressPercentage, activeJob.MonitorUrl);
            Status.EndNotify(jobsContext);

            Status.Notify(extractionsContext, null, "MonitorJob");
            //Select a job and monitor it until it is complete.  Of course there may be multiple active jobs running and 
            //you can't be sure which job you are monitoring which is the reason the preferred approach is to retain the
            //job handle from the initial request and use that to monitor results.
            var firstActiveJob = activeJobs.FirstOrDefault();
            if (firstActiveJob != null)
            {
                IAsyncJob firstActiveAsyncJob;
                do
                {
                    firstActiveAsyncJob = extractionsContext.MonitorJob(firstActiveJob.MonitorUrl);
                    Thread.Sleep(3000);
                } while (firstActiveAsyncJob.Status != JobStatus.Completed
                         && firstActiveAsyncJob.Status != JobStatus.Error
                         && firstActiveAsyncJob.Status != JobStatus.Cancelled);
                if (firstActiveJob.Status == JobStatus.Completed)
                {
                    var result = (ExtractionResult)firstActiveAsyncJob.Result;
                    Status.WriteLine("Extracted Rows: " + result.Contents.Count());
                }
            }
            Status.EndNotify(extractionsContext);
        }

        #endregion

        #region Streaming

        //DO NOT CHANGE THE EXAMPLE NAME, the name is used verbatim in the Help documentation site.
        [Example("Streaming: Download")]
        [EndToEndExample]
        public void Streaming()
        {
            /* Streaming essentially allows the API to efficiently send large files / content via HTTP. */

            var extractionsContext = ContextHelper.CreateExtractionsContext();
            Status.Notify(extractionsContext, "ReportTemplateType", "GetContentFieldTypes");
            //Retrieve the list of available fields.  You can skip this step if you already know the list of fields to extract.
            //The list of fields contains the code, name, format.  Either the code or name can be used as the fieldName when
            //adding content fields
            var availableFields = extractionsContext.GetValidContentFieldTypes(ReportTemplateTypes.EndOfDayPricing);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "Create");
            //Create the instrument list
            var instrumentList = new InstrumentList { Name = "ImmediateNonCustomized" + Suffix.New() };
            extractionsContext.InstrumentListOperations.Create(instrumentList);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "InstrumentList", "AppendIdentifiers");
            //Add instruments to the list
            var instruments = new[]
            {
                new InstrumentIdentifier { Identifier = "IBM.N", IdentifierType = IdentifierType.Ric },
                new InstrumentIdentifier { Identifier = "KO.N", IdentifierType = IdentifierType.Ric }
            };
            extractionsContext.InstrumentListOperations.AppendIdentifiers(instrumentList, instruments, false);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "ReportTemplates", "Create");
            //Define a new end of day report template
            var reportTemplate = new EndOfDayPricingReportTemplate
            {
                Name = "ImmediateNonCustomized" + Suffix.New(),
                OutputFormat = ReportOutputFormat.CommaSeparatedValues
                //NOTE: There are many other fields which can be use to customize the report template
            };

            //Add fields.  See also GetAvailableFields
            reportTemplate.ContentFields.AddRange(
                "Ask Price", "Asset Category", "Asset Category Description", "Asset ID", "Asset Status",
                "Asset Status Description", "Asset SubType", "Asset SubType Description", "Asset Type",
                "Asset Type Description", "Bid Price", "Block Trades", "CIN Code", "Common Code",
                "Currency Code", "Currency Code Description", "Currency Code Scaled", "Currency Code Scaled Description",
                "CUSIP", "Exchange Code", "Exchange Description", "Exercise Style", "Expiration Date", "File Code",
                "High Price", "Instrument ID", "Instrument ID Type", "ISIN", "Issuer OrgID", "Lot Size", "Low Price",
                "Market MIC", "MIC", "Mid Price", "Net Asset Value", "Offer Price",
                "Official Close Price", "Open Price", "Previous Close Price", "Put Call Indicator", "Quote ID",
                "RCP ID", "RIC", "Security Description", "SEDOL", "Settlement Date", "Settlement Price", "SICC",
                "Sicovam", "Strike Price", "Ticker", "Trade Date", "Trading Status", "Trading Symbol", "Turnover",
                "Underlying RIC", "Universal Ask Price", "Universal Bid Ask Date", "Universal Bid Price",
                "Usage Instrument SubType", "Usage Instrument Type", "Valoren", "Volume", "VWAP Price", "Wertpapier");

            //Create the report template
            extractionsContext.ReportTemplateOperations.Create(reportTemplate);
            Status.EndNotify(extractionsContext);

            Status.Notify(extractionsContext, "Schedules", "Create");
            //Schedule the extraction to run immediately
            var schedule = new Schedule
            {
                Name = "ImmediateNonCustomized" + Suffix.New(),
                TimeZone = TimeZoneInfo.Local.Id,
                Recurrence = ScheduleRecurrence.CreateSingleRecurrence(DateTimeOffset.UtcNow, true),
                Trigger = ScheduleTrigger.CreateImmediateTrigger(true),
                ListId = instrumentList.ListId,
                ReportTemplateId = reportTemplate.ReportTemplateId,
            };
            extractionsContext.ScheduleOperations.Create(schedule);
            Status.EndNotify(extractionsContext);

            try
            {
                Status.Notify(extractionsContext, "ReportExtraction", "GetLastExtractionForScheduleId");
                //Wait for the extraction to complete.  This involves polling GetLastExtractionForScheduleId until 
                //the returned ReportExtraction.Status is completed.
                var extraction = extractionsContext.ScheduleOperations.WaitForNextExtraction(schedule);

                //Output some extraction information
                Status.WriteLine("status: {0} / detailed status: {1} / extraction date: {2} - {3} / files = {4}",
                    extraction.Status, extraction.DetailedStatus, extraction.ExtractionStartUtc, extraction.ExtractionEndUtc,
                    extraction.Files.Count);
                Status.EndNotify(extractionsContext);

                Status.Notify(extractionsContext, "ReportExtraction", "Files");
                //Get the list of files for this extraction.  This includes the contents, notes files and RIC maintenance files.
                extractionsContext.LoadProperty(extraction, "Files");

                //Find the extract file.  Other ExtractedFileTypes are: Note, RicMaintenanceNote, Full, Partial, Other
                var extractFile = extraction.Files.FirstOrDefault(f => f.FileType == ExtractedFileType.Full);
                Status.EndNotify(extractionsContext);

                if (extractFile != null)
                {
                    Status.Notify(extractionsContext, "ReportExtraction", "GetReadStream");
                    //Output the contents of the extracted file
                    var streamResponse = extractionsContext.ExtractedFileOperations.GetReadStream(extractFile);
	                string internalStreamOutput;
                    using (var reader = new StreamReader(streamResponse.Stream))
	                {
		                Status.WriteLine(internalStreamOutput = reader.ReadToEnd());
	                }
	                Status.EndNotify(extractionsContext, internalStreamOutput); // Extract the stream output and pass it as responseBody.
                }
            }
            catch (TimeoutException ex)
            {
                Status.WriteLine("Timed out: " + ex.Message);
            }

            //Delete the extracted files, schedule, instrument list and report template.  See the "Cleanup" example for
            //the various steps involved with cleanup.  NOTE: It is a good idea to clean up the extracted files (Report Extractions)
            //once you have downloaded the results.  If the schedule is not immediate (i.e. recurring), you want to keep the 
            //instrument list, report template and schedule around.
            Cleanup(schedule, instrumentList, reportTemplate);
        }

        [Example("Streaming: Resume")]
        [EndToEndExample]
        public void StreamingResume()
        {
            /* This example demonstrates how to download the contents from a specific byte offset.
             * NOTE: Although this includes a .Net SDK example, users of the .Net SDK should never need to manually resume downloads
             * since this is automatically handled by the SDK.  The SDK returns a ResillientStream which will resume in the event
             * that the download is interrupted. */
            var standardExtractionsContext = ContextHelper.CreateStandardExtractionsContext();

            Status.Notify(standardExtractionsContext, "UserPackageDelivery", "GetAll");
            //Get package deliveries
            var packageDelivery = standardExtractionsContext.UserPackageDeliveryOperations.GetAll().FirstOrDefault();
            Status.EndNotify(standardExtractionsContext);

            Status.Notify(standardExtractionsContext, "UserPackageDelivery", "GetReadStream");
            //Get the contents of the userPackageDelivery, but only from byte offset 42.  You would only use the byteOffset
            //to resume downloading.
            using (var response = standardExtractionsContext.UserPackageDeliveryOperations.GetReadStream(packageDelivery, 42))
            using (var fileStream = File.Create("..\\" + packageDelivery.Name))
                response.Stream.CopyToAsync(fileStream).Wait();
            Status.EndNotify(standardExtractionsContext);
        }

        #endregion

        #region Validation

        //DO NOT CHANGE THE EXAMPLE NAME, the name is used verbatim in the Help documentation site.
        [Example("Validate: Validation Error (400 status code)")]
        [EndToEndExample]
        public void CreateWithValidationError()
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();
            Status.Notify(extractionsContext, "EntityList", "Create");
            try
            {
                //Create new entity list with a null name to force a validation error
                extractionsContext.EntityListOperations.Create(new EntityList { Name = null });
            }
            catch (ValidationException ex)
            {
                //Output the combined error message
                Status.WriteLine("General Message: " + ex.Message);
                //Output individual error messages.
                foreach (var message in ex.ValidationErrors)
                    Status.WriteLine("Validation Message: id ({0}), message ({1}), property ({2}), severity({3})",
                        message.Id, message.Message, message.PropertyName, message.Severity);
            }
            Status.EndNotify(extractionsContext);
        }

        #endregion

        #region Cleanup Helper Method

        // This cleanup code serves as an example of how to do a get-all for the various entity types.
        // The cleanup button is useful if you abort debug sessions while exploring these samples.  
        // Cleanup is by name prefix.  It removes all lists, reports, schedules, and file content that matches the prefix.

        public void Cleanup(Schedule schedule, InstrumentList instrumentList, ReportTemplate reportTemplate)
        {
            var extractionsContext = ContextHelper.CreateExtractionsContext();

            if (schedule != null && schedule.ScheduleId != null)
            {
                var scheduleExtractions = extractionsContext.ReportExtractionOperations.GetByScheduleId(schedule.ScheduleId);
                foreach (var reportExtraction in scheduleExtractions)
                {
                    extractionsContext.LoadProperty(reportExtraction, "Files");
                    var fileIds = reportExtraction.Files
                                                  .Where(f => f.ContentsExists)
                                                  .Select(f => f.ExtractedFileId)
                                                  .ToList();
                    if (fileIds.Count > 0)
                        extractionsContext.ExtractedFileOperations.DeleteMultiple(fileIds);
                }
                extractionsContext.ScheduleOperations.Delete(schedule);
            }

            //Delete the report template
            if (reportTemplate != null && reportTemplate.ReportTemplateId != null)
                extractionsContext.ReportTemplateOperations.Delete(reportTemplate);

            //Delete the instrument list
            if (instrumentList != null && instrumentList.ListId != null)
                extractionsContext.InstrumentListOperations.Delete(instrumentList);

        }

        #endregion
    }
}
